# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to import an Anthos clusters on bare metal API."""


from googlecloudsdk.api_lib.container.gkeonprem import bare_metal_clusters as apis
from googlecloudsdk.api_lib.container.gkeonprem import operations
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_arguments
from googlecloudsdk.command_lib.container.bare_metal import cluster_flags
from googlecloudsdk.command_lib.container.gkeonprem import constants
from googlecloudsdk.command_lib.export import util as export_util
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io
from googlecloudsdk.generated_clients.apis.gkeonprem.v1 import gkeonprem_v1_messages as messages

_EXAMPLES = """
A cluster can be imported by running:

  $ {command} NAME --source=<path-to-file>
"""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Hidden
class Import(base.Command):
  """Import an Anthos on bare metal user cluster."""

  detailed_help = {'EXAMPLES': _EXAMPLES}

  @staticmethod
  def GetSchemaPath(for_help=False):
    return export_util.GetSchemaPath(
        'gkeonprem', 'v1', 'BareMetalCluster', for_help=for_help
    )

  @staticmethod
  def Args(parser: parser_arguments.ArgumentInterceptor):
    cluster_flags.AddClusterResourceArg(parser, 'to import')
    export_util.AddImportFlags(
        parser, schema_path=Import.GetSchemaPath(for_help=True)
    )
    base.ASYNC_FLAG.AddToParser(parser)
    cluster_flags.AddValidationOnly(parser)

  def Run(self, args):
    cluster_ref = args.CONCEPTS.cluster.Parse()
    cluster_client = apis.ClustersClient()
    data = console_io.ReadFromFileOrStdin(args.source or '-', binary=False)

    bare_metal_cluster = export_util.Import(
        message_type=messages.BareMetalCluster,
        stream=data,
        schema_path=Import.GetSchemaPath(),
    )

    operation = cluster_client.CreateFromImport(
        args, bare_metal_cluster, cluster_ref
    )

    if args.async_ and not args.IsSpecified('format'):
      args.format = constants.OPERATIONS_FORMAT

    if args.validate_only:
      return

    if args.async_:
      log.CreatedResource(
          cluster_ref, 'Anthos Cluster on bare metal', args.async_
      )
      return operation
    else:
      operation_client = operations.OperationsClient()
      operation_response = operation_client.Wait(operation)
      log.CreatedResource(
          cluster_ref, 'Anthos Cluster on bare metal', args.async_
      )

      return operation_response



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'logging operations list' command."""


from googlecloudsdk.api_lib.logging import util
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core.resource import resource_projector


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.GA, base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA
)
class List(base.ListCommand):
  """List long running operations."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    parser.add_argument(
        '--location', required=True, help='Location of the operations.')
    parser.add_argument(
        '--operation-filter',
        required=True,
        help=arg_parsers.UniverseHelpText(
            default=(
                'Filter expression that specifies the operations to return.'
            ),
            universe_help='Not all operation types are supported.\n',
        ),
    )
    parser.add_argument(
        '--page-token',
        type=str,
        help=(
            'The next_page_token value returned from a previous List request,'
            ' if any.'
        ),
    )
    base.URI_FLAG.RemoveFromParser(parser)
    base.FILTER_FLAG.RemoveFromParser(parser)

    util.AddParentArgs(parser, 'operations to list')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Yields:
      A list of operations.
    """
    operation_name = util.CreateResourceName(
        util.GetParentFromArgs(args), 'locations', args.location)

    request = util.GetMessages().LoggingProjectsLocationsOperationsListRequest(
        name=operation_name,
        filter=args.operation_filter,
        pageSize=args.page_size,
        pageToken=args.page_token,
    )

    result = util.GetClient().projects_locations_operations.List(request)
    self._cancellation_requested = False
    for operation in result.operations:
      yield operation
      if not self._cancellation_requested:
        serialize_op = resource_projector.MakeSerializable(operation)
        self._cancellation_requested = serialize_op.get('metadata', {}).get(
            'cancellationRequested', '')
    if result.nextPageToken:
      yield result.nextPageToken

  def Epilog(self, resources_were_displayed):
    if self._cancellation_requested:
      log.status.Print(
          'Note: Cancellation happens asynchronously. It may take up to 10 '
          "minutes for the operation's status to change to cancelled.")


List.detailed_help = {
    'DESCRIPTION': """
        Return a list of long running operations in the given LOCATION. The
        operations were scheduled by other gcloud commands.

        For example, a CopyLogEntries operation may be scheduled by the command:
        `gcloud logging copy BUCKET_ID DESTINATION --location=LOCATION`.

        The `--operation-filter` flag is required and must specify the
        `request_type`. Supported request types include but are not limited to:
        `CopyLogEntries`, `CreateBucket` and `UpdateBucket`.

        Additional supported filter expressions include: `operation_start_time`,
        `operation_finish_time` and `operation_state`. These can be combined
        with the case-sensitive keyword `AND` between them.

        For `operation_start_time` and `operation_end_time`, the operators >=,
        >, <=, and < are supported.

        Timestamps must be in either RFC3339 or ISO8601 formats. If the
        timestamp contains a time value, then it must be quoted. For examples:
        "YYYY-MM-DDTHH:MM:SSZ", "YYYY-MM-DDTHH:MM:SS.mmmZ", "YY-MM-DD",
        "YYYY-MM-DDTHH:MM:SS-0000", "YYYY-MM-DDTHH:MM+0000", "YYYY-MM-DD",
        YYYY-MM-DD, YY-MM-DD, etc.

        The `operation_state` filter expression can be used to filter for
        operations that are in a specific state. The value can be one of the
        following: `SCHEDULED`, `WAITING_FOR_PRECONDITIONS`, `RUNNING`,
        `SUCCESS`, `FAILURE`, `CANCELLED`, `PENDING`.

        For `operation_state`, the operators = and != are supported.

        Other filter options are not supported.
        """,
    'EXAMPLES': """\
        To list CopyLogEntries operations, run:

            $ {command} --location=LOCATION --operation-filter='request_type=CopyLogEntries'

        To list CopyLogEntries operations that started after a specified time, run:

            $ {command} --location=LOCATION --operation-filter='request_type=CopyLogEntries AND operation_start_time>="2023-11-20T00:00:00Z"'

        To list CopyLogEntries operations that finished before a specified time, run:

            $ {command} --location=LOCATION --operation-filter='request_type=CopyLogEntries AND operation_finish_time<="2023-11-20T00:00:00Z"'

        To list CopyLogEntries operations that completed successfully, run:

            $ {command} --location=LOCATION --operation-filter='request_type=CopyLogEntries AND operation_state=SUCCESS'

        To list CopyLogEntries operations that have not failed, run:

            $ {command} --location=LOCATION --operation-filter='request_type=CopyLogEntries AND operation_state!=FAILURE'
        """,
}



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The gcloud Firestore databases clone command."""


import textwrap

from googlecloudsdk.api_lib.firestore import databases
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.firestore import flags
from googlecloudsdk.command_lib.firestore import util as utils
from googlecloudsdk.core import properties


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Clone(base.Command):
  """Clone a Google Cloud Firestore database from another.

  ## EXAMPLES

  To clone a database from another:

      $ {command}
      --source-database=projects/PROJECT_ID/databases/SOURCE_DATABASE
      --snapshot-time=2025-05-26T10:20:00.00Z
      --destination-database=DATABASE_ID

  To clone to a CMEK-enabled database:

      $ {command}
      --source-database=projects/PROJECT_ID/databases/SOURCE_DATABASE
      --snapshot-time=2025-05-26T10:20:00.00Z
      --destination-database=DATABASE_ID
      --encryption-type=customer-managed-encryption
      --kms-key-name=projects/PROJECT_ID/locations/LOCATION_ID/keyRings/KEY_RING_ID/cryptoKeys/CRYPTO_KEY_ID
  """

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        '--source-database',
        metavar='SOURCE_DATABASE',
        type=str,
        required=True,
        help=textwrap.dedent("""\
            The source database to clone from.

            For example, to clone from database
            source-db:

            $ {command} --source-database=projects/PROJECT_ID/databases/source-db
        """),
    )
    parser.add_argument(
        '--snapshot-time',
        metavar='SNAPSHOT_TIME',
        type=str,
        required=True,
        help=textwrap.dedent("""\
            Snapshot time at which to clone. This must be a whole minute, in the past, and not earlier than the source database's earliest_version_time.
            Additionally, if older than one hour in the past, PITR must be enabled on the source database.

            For example, to restore from snapshot `2025-05-26T10:20:00.00Z` of source database `source-db`:

            $ {command} --source-database=projects/PROJECT_ID/databases/source-db --snapshot-time=2025-05-26T10:20:00.00Z
        """),
    )
    flags.AddDestinationDatabase(parser, 'clone', 'database')
    flags.AddEncryptionConfigGroup(parser, 'database')
    flags.AddTags(parser, 'database')

  def Run(self, args):
    project = properties.VALUES.core.project.Get(required=True)
    return databases.CloneDatabase(
        project,
        args.source_database,
        args.snapshot_time,
        args.destination_database,
        self.EncryptionConfig(args),
        args.tags,
    )

  def EncryptionConfig(self, args):
    return utils.ExtractEncryptionConfig(args)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Encrypt a plaintext file using a key."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.kms import crc32c
from googlecloudsdk.command_lib.kms import e2e_integrity
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.util import files


class Encrypt(base.Command):
  r"""Encrypt a plaintext file using a key.

  Encrypts the given plaintext file using the given CryptoKey and writes the
  result to the named ciphertext file. The plaintext file must not be larger
  than 64KiB.

  If an additional authenticated data file is provided, its contents must also
  be provided during decryption. The file must not be larger than 64KiB.

  The flag `--version` indicates the version of the key to use for
  encryption. By default, the primary version is used.

  If `--plaintext-file` or `--additional-authenticated-data-file` is set to '-',
  that file is read from stdin. Similarly, if `--ciphertext-file` is set to '-',
  the ciphertext is written to stdout.

  By default, the command performs integrity verification on data sent to and
  received from Cloud KMS. Use `--skip-integrity-verification` to disable
  integrity verification.

  ## EXAMPLES
  The following command will read the file 'path/to/plaintext', encrypt it using
  the CryptoKey `frodo` with the KeyRing `fellowship` and Location `global`, and
  write the ciphertext to 'path/to/ciphertext'.

    $ {command} \
        --key=frodo \
        --keyring=fellowship \
        --location=global \
        --plaintext-file=path/to/input/plaintext \
        --ciphertext-file=path/to/output/ciphertext
  """

  @staticmethod
  def Args(parser):
    flags.AddKeyResourceFlags(parser, 'The key to use for encryption.')
    flags.AddCryptoKeyVersionFlag(parser, 'to use for encryption')
    flags.AddPlaintextFileFlag(parser, 'to encrypt')
    flags.AddCiphertextFileFlag(parser, 'to output')
    flags.AddAadFileFlag(parser)
    flags.AddSkipIntegrityVerification(parser)

  def _ReadFileOrStdin(self, path, max_bytes):
    data = console_io.ReadFromFileOrStdin(path, binary=True)
    if len(data) > max_bytes:
      raise exceptions.BadFileException(
          'The file [{0}] is larger than the maximum size of {1} bytes.'.format(
              path, max_bytes))
    return data

  def _PerformIntegrityVerification(self, args):
    return not args.skip_integrity_verification

  def _MaybeStripResourceVersion(self, req_name, resp_name):
    """Strips the trailing '/cryptoKeyVersions/xx' from Response's resource name.

    If request's resource name is a key and not a version, returns response's
    resource name with the trailing '/cryptoKeyVersions/xx' suffix stripped.
    Args:
      req_name: String.
        CloudkmsProjectsLocationsKeyRingsCryptoKeysEncryptRequest.name.
      resp_name: String. EncryptResponse.name.

    Returns:
      resp_resource_name with '/cryptoKeyVersions/xx' suffix stripped.
    """
    if req_name.find('/cryptoKeyVersions/') == -1:
      return resp_name.partition('/cryptoKeyVersions/')[0]
    else:
      return resp_name

  def _CreateEncryptRequest(self, args):
    if (args.plaintext_file == '-' and
        args.additional_authenticated_data_file == '-'):
      raise exceptions.InvalidArgumentException(
          '--plaintext-file',
          '--plaintext-file and --additional-authenticated-data-file cannot '
          'both read from stdin.')

    try:
      # The Encrypt API limits the plaintext to 64KiB.
      plaintext = self._ReadFileOrStdin(args.plaintext_file, max_bytes=65536)
    except files.Error as e:
      raise exceptions.BadFileException(
          'Failed to read plaintext file [{0}]: {1}'.format(
              args.plaintext_file, e))

    aad = None
    if args.additional_authenticated_data_file:
      try:
        # The Encrypt API limits the AAD to 64KiB.
        aad = self._ReadFileOrStdin(
            args.additional_authenticated_data_file, max_bytes=65536)

      except files.Error as e:
        raise exceptions.BadFileException(
            'Failed to read additional authenticated data file [{0}]: {1}'
            .format(args.additional_authenticated_data_file, e))

    if args.version:
      crypto_key_ref = flags.ParseCryptoKeyVersionName(args)
    else:
      crypto_key_ref = flags.ParseCryptoKeyName(args)

    messages = cloudkms_base.GetMessagesModule()

    req = messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysEncryptRequest(
        name=crypto_key_ref.RelativeName())

    # Populate request integrity fields.
    if self._PerformIntegrityVerification(args):
      plaintext_crc32c = crc32c.Crc32c(plaintext)
      # Set checksum if AAD is not provided for simpler response verification.
      aad_crc32c = crc32c.Crc32c(aad) if aad is not None else crc32c.Crc32c(b'')
      req.encryptRequest = messages.EncryptRequest(
          plaintext=plaintext,
          additionalAuthenticatedData=aad,
          plaintextCrc32c=plaintext_crc32c,
          additionalAuthenticatedDataCrc32c=aad_crc32c)
    else:
      req.encryptRequest = messages.EncryptRequest(
          plaintext=plaintext, additionalAuthenticatedData=aad)

    return req

  def _VerifyResponseIntegrityFields(self, req, resp):
    """Verifies integrity fields in EncryptResponse.

    Note: This methods assumes that self._PerformIntegrityVerification() is True
    and that all request CRC32C fields were pupolated.
    Args:
      req: messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysEncryptRequest()
        object
      resp: messages.EncryptResponse() object.

    Returns:
      Void.
    Raises:
      e2e_integrity.ServerSideIntegrityVerificationError if the server reports
      request integrity verification error.
      e2e_integrity.ClientSideIntegrityVerificationError if response integrity
      verification fails.
    """

    # Verify resource name.
    # Strip version from resp.name if --key was provided.
    resp_name = self._MaybeStripResourceVersion(req.name, resp.name)
    if req.name != resp_name:
      raise e2e_integrity.ResourceNameVerificationError(
          e2e_integrity.GetResourceNameMismatchErrorMessage(
              req.name, resp_name))

    # plaintext_crc32c was verified server-side.
    if not resp.verifiedPlaintextCrc32c:
      raise e2e_integrity.ClientSideIntegrityVerificationError(
          e2e_integrity.GetRequestToServerCorruptedErrorMessage())

    # additional_authenticated_data_crc32c was verified server-side.
    if not resp.verifiedAdditionalAuthenticatedDataCrc32c:
      raise e2e_integrity.ClientSideIntegrityVerificationError(
          e2e_integrity.GetRequestToServerCorruptedErrorMessage())

    # Verify ciphertext checksum.
    if not crc32c.Crc32cMatches(resp.ciphertext, resp.ciphertextCrc32c):
      raise e2e_integrity.ClientSideIntegrityVerificationError(
          e2e_integrity.GetResponseFromServerCorruptedErrorMessage())

  def Run(self, args):
    client = cloudkms_base.GetClientInstance()
    req = self._CreateEncryptRequest(args)
    try:
      resp = client.projects_locations_keyRings_cryptoKeys.Encrypt(req)
    # Intercept INVALID_ARGUMENT errors related to checksum verification, to
    # present a user-friendly message. All other errors are surfaced as-is.
    except apitools_exceptions.HttpBadRequestError as error:
      e2e_integrity.ProcessHttpBadRequestError(error)

    if self._PerformIntegrityVerification(args):
      self._VerifyResponseIntegrityFields(req, resp)

    try:
      log.WriteToFileOrStdout(
          args.ciphertext_file, resp.ciphertext, binary=True, overwrite=True)
    except files.Error as e:
      raise exceptions.BadFileException(e)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The meta cache list command."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.meta import cache_util
from googlecloudsdk.core import log
from googlecloudsdk.core.cache import exceptions as cache_exceptions


class List(base.ListCommand):
  """List the tables or table contents in a persistent cache."""

  @staticmethod
  def Args(parser):
    cache_util.AddCacheFlag(parser)
    parser.add_argument(
        'tables',
        nargs='*',
        help=('The table names or name patterns to list, where `?` matches any '
              'character and ```*``` matches any string of zero or more '
              'characters. If omitted then a table of all tables is '
              'displayed.'))

  def Run(self, args):
    with cache_util.GetCache(args.cache) as cache:
      log.info('cache name {}'.format(cache.name))

      if args.tables:
        names = [name for pattern in args.tables
                 for name in cache.Select(pattern)]
        if not names:
          raise cache_util.NoTablesMatched('No tables matched [{}].'.format(
              ','.join(args.tables)))
        if not args.IsSpecified('format'):
          args.format = 'json'
        results = []
        for name in names:
          try:
            table = cache.Table(name, create=False)
            results.append({
                'name': table.name,
                'data': table.Select(ignore_expiration=False)
            })
          except cache_exceptions.Error as e:
            log.warning(e)
        return results

      if not args.IsSpecified('format'):
        args.format = ('table[box](name, columns:label=COL, keys:label=KEY, '
                       'timeout, is_expired:label=EXPIRED)')
      names = cache.Select()
      return [cache.Table(name=name, create=False) for name in sorted(names)]



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Add IAM Policy Binding for EkmConfig."""


from googlecloudsdk.api_lib.cloudkms import iam
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.kms import resource_args


class AddIamPolicyBinding(base.Command):
  """Add IAM policy binding to an EkmConfig.

  Adds a policy binding to the IAM policy of a kms EkmConfig. A binding consists
  of at least one member, a role, and an optional condition.

  ## EXAMPLES
  To add an IAM policy binding for the role of 'roles/editor' for the user
  `test-user@gmail.com` on the EkmConfig with location `us-central1`, run:

    $ {command} --location='us-central1' --member='user:test-user@gmail.com'
    --role='roles/editor'

  To add an IAM policy binding which expires at the end of the year 2022 for the
  role of 'roles/editor' and the user `test-user@gmail.com` and location
  `us-central1`, run:

    $ {command} --location='us-central1' --member='user:test-user@gmail.com'
    --role='roles/editor' --condition='expression=request.time <
    timestamp("2023-01-01T00:00:00Z"),title=expires_end_of_2022,description=Expires
    at midnight on 2022-12-31'

  See https://cloud.google.com/iam/docs/managing-policies for details of
  policy role and member types.
  """

  @staticmethod
  def Args(parser):
    resource_args.AddKmsLocationResourceArgForKMS(parser, True, '--location')
    iam_util.AddArgsForAddIamPolicyBinding(parser, add_condition=True)

  def Run(self, args):
    location_ref = args.CONCEPTS.location.Parse()
    ekm_config_name = 'projects/{0}/locations/{1}/ekmConfig'.format(
        location_ref.projectsId, location_ref.locationsId)
    result = iam.AddPolicyBindingToEkmConfig(ekm_config_name, args.member,
                                             args.role)
    iam_util.LogSetIamPolicy(ekm_config_name, 'EkmConfig')
    return result



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to delete the specified Kafka source."""


from googlecloudsdk.api_lib.eventarc import kafka_sources
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.eventarc import flags
from googlecloudsdk.core.console import console_io

_DETAILED_HELP = {
    'DESCRIPTION': '{description}',
    'EXAMPLES': """ \
        To delete the Kafka source `my-kafka-source` in location `us-central1`, run:

          $ {command} my-kafka-source --location=us-central1
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.DefaultUniverseOnly
class Delete(base.DeleteCommand):
  """Delete an Eventarc Kafka source."""

  detailed_help = _DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    flags.AddKafkaSourceResourceArg(
        parser, 'Kafka source to delete.', required=True
    )
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    """Run the delete command."""
    client = kafka_sources.KafkaSourceClientV1()
    kafka_source_ref = args.CONCEPTS.kafka_source.Parse()

    console_io.PromptContinue(
        message=(
            'The following Kafka source will be deleted.\n'
            '[{name}] in location [{location}]'.format(
                name=kafka_source_ref.kafkaSourcesId,
                location=kafka_source_ref.locationsId,
            )
        ),
        throw_if_unattended=True,
        cancel_on_no=True,
    )
    operation = client.Delete(kafka_source_ref)

    if args.async_:
      return operation
    return client.WaitFor(operation, 'Deleting', kafka_source_ref)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for compute composite-health-checks commands."""

from googlecloudsdk.command_lib.compute import flags


def CompositeHealthCheckArgument(required=True, plural=False):
  return flags.ResourceArgument(
      name='COMPOSITE_HEALTH_CHECK',
      resource_name='composite health check',
      regional_collection='compute.regionCompositeHealthChecks',
      required=required,
      plural=plural,
  )



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update the specified enrollment."""


from googlecloudsdk.api_lib.eventarc import enrollments
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.eventarc import flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log

_DETAILED_HELP = {
    'DESCRIPTION': '{description}',
    'EXAMPLES': """ \
        To update the enrollment `my-enrollment` with a new CEL expression `message.type == 'google.cloud.pubsub.topic.v1.messagePublished'`, run:

          $ {command} my-enrollment --location=us-central1 --cel-match="message.type == 'google.cloud.pubsub.topic.v1.messagePublished'"
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Update(base.UpdateCommand):
  """Update an Eventarc enrollment."""

  detailed_help = _DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    flags.AddUpdateEnrollmentResourceArgs(parser)
    flags.AddCelMatchArg(parser, required=False)
    labels_util.AddUpdateLabelsFlags(parser)

    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    """Run the update command."""
    client = enrollments.EnrollmentClientV1()
    enrollment_ref = args.CONCEPTS.enrollment.Parse()

    log.debug(
        'Updating enrollment {} for project {} in location {}'.format(
            enrollment_ref.enrollmentsId,
            enrollment_ref.projectsId,
            enrollment_ref.locationsId,
        )
    )

    original_enrollment = client.Get(enrollment_ref)
    labels_update_result = labels_util.Diff.FromUpdateArgs(args).Apply(
        client.LabelsValueClass(), original_enrollment.labels
    )

    update_mask = client.BuildUpdateMask(
        cel_match=args.IsSpecified('cel_match'),
        destination=args.IsSpecified('destination_pipeline'),
        labels=labels_update_result.needs_update,
    )

    operation = client.Patch(
        enrollment_ref,
        client.BuildEnrollment(
            enrollment_ref=enrollment_ref,
            cel_match=args.cel_match,
            message_bus_ref=None,
            destination_ref=args.CONCEPTS.destination_pipeline.Parse(),
            labels=labels_update_result.GetOrNone(),
        ),
        update_mask,
    )

    if args.async_:
      return operation
    return client.WaitFor(operation, 'Updating', enrollment_ref)



#!/usr/bin/env python
"""The BigQuery CLI cancel command."""

from typing import Optional

from absl import flags

import bq_flags
from clients import client_job
from clients import utils as bq_client_utils
from frontend import bigquery_command
from frontend import bq_cached_client
from frontend import utils as frontend_utils
from utils import bq_id_utils

# These aren't relevant for user-facing docstrings:
# pylint: disable=g-doc-return-or-yield
# pylint: disable=g-doc-args


class Cancel(bigquery_command.BigqueryCmd):
  """Attempt to cancel the specified job if it is running."""

  usage = """cancel [--nosync] [<job_id>]"""
  command = 'cancel'

  def __init__(self, name: str, fv: flags.FlagValues):
    super(Cancel, self).__init__(name, fv)
    self._ProcessCommandRc(fv)

  def RunWithArgs(self, job_id: str = '') -> Optional[int]:
    # pylint: disable=g-doc-exception
    """Request a cancel and waits for the job to be cancelled.

    Requests a cancel and then either:
    a) waits until the job is done if the sync flag is set [default], or
    b) returns immediately if the sync flag is not set.
    Not all job types support a cancel, an error is returned if it cannot be
    cancelled. Even for jobs that support a cancel, success is not guaranteed,
    the job may have completed by the time the cancel request is noticed, or
    the job may be in a stage where it cannot be cancelled.

    Examples:
      bq cancel job_id  # Requests a cancel and waits until the job is done.
      bq --nosync cancel job_id  # Requests a cancel and returns immediately.

    Arguments:
      job_id: Job ID to cancel.
    """
    client = bq_cached_client.Client.Get()
    job_reference_dict = dict(
        bq_client_utils.GetJobReference(
            id_fallbacks=client,
            identifier=job_id,
            default_location=bq_flags.LOCATION.value,
        )
    )
    job = client_job.CancelJob(
        bqclient=client,
        job_id=job_reference_dict['jobId'],
        location=job_reference_dict['location'],
    )
    frontend_utils.PrintObjectInfo(
        job,
        bq_id_utils.ApiClientHelper.JobReference.Create(**job['jobReference']),
        custom_format='show',
    )
    status = job['status']
    if status['state'] == 'DONE':
      if (
          'errorResult' in status
          and 'reason' in status['errorResult']
          and status['errorResult']['reason'] == 'stopped'
      ):
        print('Job has been cancelled successfully.')
      else:
        print('Job completed before it could be cancelled.')
    else:
      print('Job cancel has been requested.')
    return 0



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update a Cloud NetApp Volume Replication."""


from googlecloudsdk.api_lib.netapp.volumes.replications import client as replications_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.netapp import flags
from googlecloudsdk.command_lib.netapp.volumes.replications import flags as replications_flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.command_lib.util.concepts import concept_parsers

from googlecloudsdk.core import log


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Update(base.UpdateCommand):
  """Update a Cloud NetApp Volume Replication."""

  _RELEASE_TRACK = base.ReleaseTrack.GA

  detailed_help = {
      'DESCRIPTION': """\
          Update a Cloud NetApp Volume Replication and its specified parameters.
          """,
      'EXAMPLES': """\
          The following command updates a Replication named NAME and its specified parameters:

              $ {command} NAME --location=us-central1 --volume=vol1 --replication-schedule=EVERY_5_MINUTES --description="new description" --cluster-location= us-central1
          """,
  }

  @staticmethod
  def Args(parser):
    """Add args for updating a Replication."""
    concept_parsers.ConceptParser(
        [flags.GetReplicationPresentationSpec('The Replication to update.')]
    ).AddToParser(parser)
    replications_flags.AddReplicationVolumeArg(parser)
    replications_flags.AddReplicationReplicationScheduleArg(
        parser, required=False
    )
    replications_flags.AddReplicationClusterLocationArg(parser)
    flags.AddResourceAsyncFlag(parser)
    flags.AddResourceDescriptionArg(parser, 'Replication')
    labels_util.AddUpdateLabelsFlags(parser)

  def Run(self, args):
    """Update a Cloud NetApp Volume Replication in the current project."""
    replication_ref = args.CONCEPTS.replication.Parse()

    client = replications_client.ReplicationsClient(self._RELEASE_TRACK)
    labels_diff = labels_util.Diff.FromUpdateArgs(args)
    original_replication = client.GetReplication(replication_ref)

    # Update labels
    if labels_diff.MayHaveUpdates():
      labels = labels_diff.Apply(
          client.messages.Replication.LabelsValue, original_replication.labels
      ).GetOrNone()
    else:
      labels = None

    replication_schedule_enum = (
        replications_flags.GetReplicationReplicationScheduleEnumFromArg(
            args.replication_schedule, client.messages
        )
    )

    replication = client.ParseUpdatedReplicationConfig(
        original_replication, description=args.description, labels=labels,
        replication_schedule=replication_schedule_enum,
        cluster_location=args.cluster_location,
    )

    updated_fields = []
    # Add possible updated replication fields.
    # TODO(b/243601146) add config mapping and separate config file for update
    if args.IsSpecified('description'):
      updated_fields.append('description')
    if (
        args.IsSpecified('update_labels')
        or args.IsSpecified('remove_labels')
        or args.IsSpecified('clear_labels')
    ):
      updated_fields.append('labels')
    if args.IsSpecified('replication_schedule'):
      updated_fields.append('replication_schedule')

    if args.IsSpecified('cluster_location'):
      updated_fields.append('cluster_location')

    update_mask = ','.join(updated_fields)
    result = client.UpdateReplication(
        replication_ref, replication, update_mask, args.async_
    )
    if args.async_:
      command = 'gcloud {} netapp volumes replications list'.format(
          self.ReleaseTrack().prefix
      )
      log.status.Print(
          'Check the status of the updated replication by listing all'
          ' replications:\n  $ {} '.format(command)
      )
    return result


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class UpdateBeta(Update):
  """Update a Cloud NetApp Volume Replication."""

  _RELEASE_TRACK = base.ReleaseTrack.BETA


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAlpha(UpdateBeta):
  """Update a Cloud NetApp Volume Replication."""

  _RELEASE_TRACK = base.ReleaseTrack.ALPHA



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a experiment in the Project/Location."""


from googlecloudsdk.api_lib.fault_injection import experiments
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.fault_injection import flags


_DETAILED_HELP = {
    'DESCRIPTION': '{description}',
    'EXAMPLES': """ \
        To create a experiment with the id `my-experiment` and file path `my/path/to/yaml`, run:

          $ {command} my-experiment --file=my/path/to/yaml
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Create(base.CreateCommand):
  """Command to create a experiment in the Project/Location."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddCreateExperimentFlags(parser)

  def Run(self, args):
    """Run the create command."""
    client = experiments.ExperimentsClient()
    exp_ref = args.CONCEPTS.experiment.Parse()
    parent_ref = exp_ref.Parent()
    if not exp_ref.Name():
      raise exceptions.InvalidArgumentException(
          'experiment', 'experiment id must be non-empty.'
      )
    if not args.file:
      raise exceptions.InvalidArgumentException(
          'experiment', 'file path must be non-empty.'
      )
    return client.Create(
        experiment=exp_ref.Name(),
        experiment_config=args.file,
        parent=parent_ref.RelativeName(),
    )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list Cloud Dataflow snapshots."""


from googlecloudsdk.api_lib.dataflow import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataflow import dataflow_util
from googlecloudsdk.command_lib.dataflow import snapshot_utils
from googlecloudsdk.core import properties


class List(base.Command):
  """List all Cloud Dataflow snapshots in a project in the specified region, optionally filtered by job ID."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To list all Cloud Dataflow snapshots in the us-central1 region, run:

            $ {command} --region=us-central1

          To list all Cloud Dataflow snapshots for a job, run:

            $ {command} --job-id=JOB_ID --region=JOB_REGION
          """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: argparse.ArgumentParser to register arguments with.
    """
    snapshot_utils.ArgsForListSnapshot(parser)

  def Run(self, args):
    """Runs the command.

    Args:
      args: The arguments that were provided to this command invocation.

    Returns:
      A Snapshot message.
    """
    return apis.Snapshots.List(
        job_id=args.job_id,
        project_id=properties.VALUES.core.project.GetOrFail(),
        region_id=dataflow_util.GetRegion(args))



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command to disable Config Delivery Argo CD Feature on a membership."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet import resources
from googlecloudsdk.command_lib.container.fleet.features import base as features_base


@base.DefaultUniverseOnly
class Disable(features_base.DisableCommand):
  """Disable Config Delivery Argo CD Feature on a membership.

  This command disables Config Delivery Argo CD Feature on a membership.

  ## EXAMPLES

  To disable the Config Delivery Argo CD Feature on a membership, run:

    $ {command} --config-membership=CONFIG_MEMBERSHIP
  """

  feature_name = 'configdeliveryargocd'

  @classmethod
  def Args(cls, parser):
    resources.AddMembershipResourceArg(
        parser, flag_override='--config-membership')

  def Run(self, args):
    # Get the feature to ensure the feature is enabled
    # Error will be raised if feature is not enabled
    self.GetFeature()

    config_membership = features_base.ParseMembership(
        args, prompt=True, flag_override='config_membership')

    membership_specs = {}
    membership_specs[config_membership] = self.messages.MembershipFeatureSpec(
        configDeliveryArgoCd=None
    )

    feature = self.messages.Feature(
        membershipSpecs=self.hubclient.ToMembershipSpecs(membership_specs)
    )

    feature = self.Update(['membership_specs'], feature)

    # Disable the feature if feature is no longer enabled for any memberships
    if feature.membershipSpecs is None:
      return self.Disable(False)

    return feature



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Destroy a version."""


from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kms import flags


class Destroy(base.UpdateCommand):
  """Schedule a version to be destroyed.

  Schedules the given version for destruction in 24 hours.

  After that time period passes it is automatically destroyed. Once
  destroyed, the key material is removed but the version number can
  not be reused.

  Only versions which are Enabled or Disabled can be Scheduled
  for destruction.

  ## EXAMPLES

  The following command schedules version 9 of key `frodo` within
  keyring `fellowship` and location `us-east1` for destruction:

    $ {command} 9 --location=us-east1 --keyring=fellowship --key=frodo
  """

  @staticmethod
  def Args(parser):
    flags.AddKeyVersionResourceArgument(parser, 'to destroy')

  def Run(self, args):
    # pylint: disable=line-too-long
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()

    version_ref = flags.ParseCryptoKeyVersionName(args)
    req = messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysCryptoKeyVersionsDestroyRequest(
        name=version_ref.RelativeName())

    ckv = client.projects_locations_keyRings_cryptoKeys_cryptoKeyVersions
    return ckv.Destroy(req)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud monitoring snoozes describe` command."""


from googlecloudsdk.api_lib.monitoring import snoozes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.monitoring import resource_args


class Describe(base.DescribeCommand):
  """Describe a snooze."""

  detailed_help = {
      'EXAMPLES': """\
          To describe a snooze, run:

            $ {command} MY-SNOOZE

          To describe a snooze in JSON, run:

            $ {command} MY-SNOOZE --format=json

          To describe a snooze contained within a specific project, run:

            $ {command} MY-SNOOZE --project=MY-PROJECT

          To describe a snooze with a fully qualified snooze ID, run:

            $ {command} projects/MY-PROJECT/snoozes/MY-SNOOZE
       """
  }

  @staticmethod
  def Args(parser):
    resources = [
        resource_args.CreateSnoozeResourceArg('to be described.')]
    resource_args.AddResourceArgs(parser, resources)

  def Run(self, args):
    client = snoozes.SnoozeClient()
    snooze_ref = args.CONCEPTS.snooze.Parse()

  # TODO(b/271427290): Replace 500 with 404 in api
    result = client.Get(snooze_ref)
    return result



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update a key."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kms import exceptions as kms_exceptions
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.command_lib.kms import maps
from googlecloudsdk.command_lib.kms import resource_args
from googlecloudsdk.command_lib.util.args import labels_util


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Update(base.UpdateCommand):
  r"""Update a key.

  1. Update the rotation schedule for the given key.

  Updates the rotation schedule for the given key. The schedule
  automatically creates a new primary version for the key
  according to `next-rotation-time` and `rotation-period` flags.

  Flag `next-rotation-time` must be in ISO 8601 or RFC3339 format,
  and `rotation-period` must be in the form INTEGER[UNIT], where units
  can be one of seconds (s), minutes (m), hours (h) or days (d).

  Key rotations performed manually via `update-primary-version` and the
  version `create` do not affect the stored `next-rotation-time`.

  2. Remove the rotation schedule for the given key with
  `remove-rotation-schedule` flag.

  3. Update/Remove the labels for the given key with `update-labels` and/or
  `remove-labels` flags.

  4. Update the primary version for the given key with `primary-version` flag.

  5. Update the Key Access Justifications policy for the given key with
  `allowed-access-reasons` flag to allow specified reasons. The key must be
  enrolled in Key Access Justifications to use this flag.

  6. Remove the Key Access Justifications policy for the given key with
  `remove-key-access-justifications-policy` flag. The key must be enrolled in
  Key Access Justifications to use this flag.

  7. Update the Key Access Justifications policy for the given key with
  `allowed_access_reasons` flag to allow zero access reasons. This effectively
  disables the key, because a policy is configured to reject all access reasons.
  The key must be enrolled in Key Access Justifications to use this flag.

  ## EXAMPLES

  The following command sets a 30 day rotation period for the key
  named `frodo` within the keyring `fellowship` and location `global`
  starting at the specified time:

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --rotation-period=30d \
        --next-rotation-time=2017-10-12T12:34:56.1234Z

  The following command removes the rotation schedule for the key
  named `frodo` within the keyring `fellowship` and location `global`:

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --remove-rotation-schedule

  The following command updates the labels value for the key
  named `frodo` within the keyring `fellowship` and location `global`. If the
  label key does not exist at the time, it will be added:

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --update-labels=k1=v1

  The following command removes labels k1 and k2 from the key
  named `frodo` within the keyring `fellowship` and location `global`:

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --remove-labels=k1,k2

  The following command updates the primary version for the key
  named `frodo` within the keyring `fellowship` and location `global`:

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --primary-version=1

  The following command updates the default algorithm for the key named `frodo`
  within the keyring `fellowship` and location `global`, assuming the key
  originally has purpose 'asymmetric-encryption' and algorithm
  'rsa-decrypt-oaep-2048-sha256':

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --default-algorithm=rsa-decrypt-oaep-4096-sha256

  The following command updates the Key Access Justifications policy for the key
  named `frodo` within the keyring ``fellowship'' and location ``global'' to
  allow only ``customer-initiated-access'' and
  ``google-initiated-system-operation'':

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --allowed-access-reasons=customer-initiated-access,google-initiated-system-operation

  The following command removes the Key Access Justifications policy for the key
  named `frodo` within the keyring ``fellowship'' and location ``global'', which
  results in all access reasons being allowed:

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --remove-key-access-justifications-policy

  The following command updates the Key Access Justifications policy for the key
  named `frodo` within the keyring ``fellowship'' and location ``global'' to
  allow only zero access reasons, effectively disabling the key:

    $ {command} frodo \
        --location=global \
        --keyring=fellowship \
        --allowed-access-reasons=
  """

  @staticmethod
  def Args(parser):
    resource_args.AddKmsKeyResourceArgForKMS(parser, True, 'key')
    flags.AddRotationPeriodFlag(parser)
    flags.AddNextRotationTimeFlag(parser)
    flags.AddRemoveRotationScheduleFlag(parser)
    flags.AddCryptoKeyPrimaryVersionFlag(parser, 'to make primary')
    labels_util.AddUpdateLabelsFlags(parser)
    flags.AddDefaultAlgorithmFlag(parser)
    flags.AddAllowedAccessReasonsFlag(parser)
    flags.AddRemoveKeyAccessJustificationsPolicyFlag(parser)

  def ProcessFlags(self, args):
    fields_to_update = []

    labels_diff = labels_util.Diff.FromUpdateArgs(args)
    if labels_diff.MayHaveUpdates():
      fields_to_update.append('labels')
    if args.remove_rotation_schedule:
      if args.rotation_period or args.next_rotation_time:
        raise kms_exceptions.ArgumentError(
            'You cannot set and remove rotation schedule at the same time.')
      fields_to_update.append('rotationPeriod')
      fields_to_update.append('nextRotationTime')
    if args.rotation_period:
      fields_to_update.append('rotationPeriod')
    if args.next_rotation_time:
      fields_to_update.append('nextRotationTime')
    if args.default_algorithm:
      fields_to_update.append('versionTemplate.algorithm')
    if (
        args.allowed_access_reasons is not None
        and args.remove_key_access_justifications_policy
    ):
      raise kms_exceptions.ArgumentError(
          'You cannot set and remove a Key Access Justifications policy at the '
          'same time.'
      )
    if (
        args.allowed_access_reasons is not None
        or args.remove_key_access_justifications_policy
    ):
      fields_to_update.append('keyAccessJustificationsPolicy')

    # Raise an exception when no update field is specified.
    if not args.primary_version and not fields_to_update:
      raise kms_exceptions.UpdateError(
          'At least one of --primary-version or --update-labels or '
          '--remove-labels or --clear-labels or --rotation-period or '
          '--next-rotation-time or --remove-rotation-schedule or '
          '--default-algorithm or --allowed-access-reasons or '
          '--remove-key-access-justifications-policy must be specified.'
      )

    return fields_to_update

  def UpdatePrimaryVersion(self, args):
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()
    crypto_key_ref = args.CONCEPTS.key.Parse()
    req = messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysUpdatePrimaryVersionRequest(  # pylint: disable=line-too-long
        name=crypto_key_ref.RelativeName(),
        updateCryptoKeyPrimaryVersionRequest=(
            messages.UpdateCryptoKeyPrimaryVersionRequest(
                cryptoKeyVersionId=args.primary_version)))

    try:
      response = client.projects_locations_keyRings_cryptoKeys.UpdatePrimaryVersion(  # pylint: disable=line-too-long
          req)
    except apitools_exceptions.HttpError:
      return None

    return response

  def UpdateOthers(self, args, crypto_key, fields_to_update):
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()
    crypto_key_ref = args.CONCEPTS.key.Parse()

    labels_update = labels_util.Diff.FromUpdateArgs(args).Apply(
        messages.CryptoKey.LabelsValue, crypto_key.labels)

    if labels_update.needs_update:
      new_labels = labels_update.labels
    else:
      new_labels = crypto_key.labels

    req = messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysPatchRequest(
        name=crypto_key_ref.RelativeName(),
        cryptoKey=messages.CryptoKey(
            labels=new_labels))
    req.updateMask = ','.join(fields_to_update)
    flags.SetNextRotationTime(args, req.cryptoKey)
    flags.SetRotationPeriod(args, req.cryptoKey)
    if args.default_algorithm:
      valid_algorithms = maps.VALID_ALGORITHMS_MAP[crypto_key.purpose]
      if args.default_algorithm not in valid_algorithms:
        raise kms_exceptions.UpdateError(
            'Update failed: Algorithm {algorithm} is not valid. Here are the '
            'valid algorithm(s) for purpose {purpose}: {all_algorithms}'.format(
                algorithm=args.default_algorithm,
                purpose=crypto_key.purpose,
                all_algorithms=', '.join(valid_algorithms)))
      req.cryptoKey.versionTemplate = messages.CryptoKeyVersionTemplate(
          algorithm=maps.ALGORITHM_MAPPER.GetEnumForChoice(
              args.default_algorithm))
    if not args.remove_key_access_justifications_policy:
      flags.SetKeyAccessJustificationsPolicy(args, req.cryptoKey)

    try:
      response = client.projects_locations_keyRings_cryptoKeys.Patch(req)
    except apitools_exceptions.HttpError:
      return None

    return response

  def HandleErrors(self, args, set_primary_version_succeeds,
                   other_updates_succeed, fields_to_update):
    """Handles various errors that may occur during any update stage.

    Never returns without an exception.

    Args:
      args: Input arguments.
      set_primary_version_succeeds: True if the primary verion is updated
        successfully.
      other_updates_succeed: True if all other updates (besides primary verions)
        is updated successfully.
      fields_to_update: A list of fields to be updated.

    Raises:
      ToolException: An exception raised when there is error during any update
      stage.
    """
    err = 'An Error occurred:'
    if not set_primary_version_succeeds:
      err += " Failed to update field 'primaryVersion'."
    elif args.primary_version:
      err += " Field 'primaryVersion' was updated."
    if not other_updates_succeed:
      err += " Failed to update field(s) '{}'.".format(
          "', '".join(fields_to_update))
    elif fields_to_update:
      err += " Field(s) '{}' were updated.".format(
          "', '".join(fields_to_update))
    raise kms_exceptions.UpdateError(err)

  def Run(self, args):
    """Updates the relevant fields (of a CryptoKey) from the flags."""

    # Check the flags and raise an exception if any check fails.
    fields_to_update = self.ProcessFlags(args)

    # Try to get the cryptoKey and raise an exception if the key doesn't exist.
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()
    crypto_key_ref = args.CONCEPTS.key.Parse()
    crypto_key = client.projects_locations_keyRings_cryptoKeys.Get(
        messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysGetRequest(
            name=crypto_key_ref.RelativeName()))

    # Try to update the key's primary version.
    set_primary_version_succeeds = True
    if args.primary_version:
      response = self.UpdatePrimaryVersion(args)
      if response:
        crypto_key = response  # If call succeeds, update the crypto_key.
      else:
        set_primary_version_succeeds = False

    # Try other updates.
    other_updates_succeed = True
    if fields_to_update:
      response = self.UpdateOthers(args, crypto_key, fields_to_update)
      if response:
        crypto_key = response  # If call succeeds, update the crypto_key.
      else:
        other_updates_succeed = False

    if not set_primary_version_succeeds or not other_updates_succeed:
      self.HandleErrors(args, set_primary_version_succeeds,
                        other_updates_succeed, fields_to_update)
    else:
      return crypto_key



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to describe a shared template revision."""


import copy
import re

from googlecloudsdk.api_lib.design_center import constants
from googlecloudsdk.api_lib.design_center import shared_template_revisions as apis
from googlecloudsdk.api_lib.design_center import utils as api_lib_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import properties

_DETAILED_HELP = {
    'DESCRIPTION': 'Describe a shared template revision.',
    'EXAMPLES': """ \
        To describe the shared template revision `my-revision` in shared template `my-shared-template`, space `my-space`, project `my-project` and location `us-central1`, run:

          $ {command} my-revision --shared-template=my-shared-template --space=my-space --project=my-project --location=us-central1

        Or run:

          $ {command} projects/my-project/locations/us-central1/spaces/my-space/sharedTemplates/my-shared-template/revisions/my-revision

        To describe a shared template revision `my-revision` in shared template `google-template` from the Google Catalog and location `us-central1`, run:

          $ {command} my-revision --shared-template=google-template --google-catalog --location=us-central1
        """,
    'API REFERENCE': constants.API_REFERENCE_ALPHA,
}

_DETAILED_HELP_GA = copy.deepcopy(_DETAILED_HELP)
_DETAILED_HELP_GA['API REFERENCE'] = constants.API_REFERENCE_GA

_BASE_ERROR_MESSAGE = 'Error parsing [revision].'

_LOCATION_NOT_SPECIFIED_MESSAGE = (
    'The [revision] resource is not properly specified.\n'
    'Failed to find attribute [location]. The attribute can be set in the'
    ' following ways:\n'
    '- provide the argument `revision` on the command line with a fully'
    ' specified name\n'
    '- provide the argument `--location` on the command line'
)

_PROJECT_NOT_SPECIFIED_MESSAGE = (
    'The [revision] resource is not properly specified.\n'
    'Failed to find attribute [project]. [project] must be specified unless'
    ' using --google-catalog, a full resource name, or the core/project'
    ' property is set. The attribute can be set in the following'
    ' ways:\n'
    '- provide the argument `revision` on the command line with a fully'
    ' specified name\n'
    '- provide the argument `--project` on the command line\n'
    '- set the property `core/project`'
)

_SPACE_NOT_SPECIFIED_MESSAGE = (
    'The [revision] resource is not properly specified.\n'
    'Failed to find attribute [space]. [space] must be specified unless'
    ' using --google-catalog or a full resource name. The attribute can'
    ' be set in the following ways:\n'
    '- provide the argument `revision` on the command line with a fully'
    ' specified name\n'
    '- provide the argument `--space` on the command line'
)

_SHARED_TEMPLATE_NOT_SPECIFIED_MESSAGE = (
    'The [revision] resource is not properly specified.\n'
    'Failed to find attribute [shared-template]. The attribute can'
    ' be set in the following ways:\n'
    '- provide the argument `revision` on the command line with a fully'
    ' specified name\n'
    '- provide the argument `--shared-template` on the command line'
)

_REVISION_HELP_TEXT = (
    'ID of the revision or fully qualified identifier for the'
    ' sharedTemplateRevision. Format:'
    ' projects/$project/locations/$location/spaces/$space/sharedTemplates/$sharedTemplate/revisions/$revision\n'
    ' To set the revision attribute:\n'
    '  * provide the fully qualified identifier `revision` on the command'
    ' line;\n'
    '  * provide the argument `revision` which represents the revision id'
    ' and the other arguments `--shared-template`, `--location`, `--project`,'
    ' `--space` or `--google-catalog` on the command line.'
)

_SHARED_TEMPLATE_HELP_TEXT = (
    'The sharedTemplate id of the revision resource.\n\n'
    'To set the shared-template attribute:\n'
    '  * provide the argument `revision` on the command line with a fully'
    ' specified name;\n'
    '  * provide the argument `--shared-template` on the command line.'
)

_LOCATION_HELP_TEXT = (
    'The location id of the revision resource.\n\n'
    'To set the location attribute:\n'
    '  * provide the argument `revision` on the command line with a fully'
    ' specified name;\n'
    '  * provide the argument `--location` on the command line.'
)

_PROJECT_HELP_TEXT = (
    'The project id of the revision resource.\n\n'
    'To set the project attribute:\n'
    '  * provide the argument `revision` on the command line with a fully'
    ' specified name;\n'
    '  * provide the argument `--project` on the command line;\n'
    '  * set the property `core/project`.'
)

_SPACE_HELP_TEXT = (
    'The space id of the revision resource.\n\n'
    'To set the space attribute:\n'
    '  * provide the argument `revision` on the command line with a fully'
    ' specified name;\n'
    '  * provide the argument `--space` on the command line.'
)

_GOOGLE_CATALOG_HELP_TEXT = (
    'If provided, describes a shared template revision from the Google Catalog.'
    ' This sets the project to "gcpdesigncenter" and space to'
    ' "googlespace".'
)


class SharedTemplateRevisionResourceError(core_exceptions.Error):
  """Exception for errors related to SharedTemplateRevision resources."""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.UniverseCompatible
class DescribeAlpha(base.DescribeCommand):
  """Describe a shared template revision."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    # Positional argument for revision
    parser.add_argument(
        'revision',
        metavar='REVISION',
        help=_REVISION_HELP_TEXT,
    )
    # Shared template flag
    parser.add_argument(
        '--shared-template',
        required=False,
        help=_SHARED_TEMPLATE_HELP_TEXT,
    )
    # Location flag
    parser.add_argument(
        '--location',
        required=False,
        help=_LOCATION_HELP_TEXT,
    )

    # Argument group for --project/--space or --google-catalog
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '--google-catalog',
        action='store_true',
        help=_GOOGLE_CATALOG_HELP_TEXT,
    )
    # Nested group for --project and --space
    project_space_group = group.add_argument_group(
        'Specify --project and --space for custom shared templates.'
    )
    project_space_group.add_argument(
        '--project',
        required=False,
        help=_PROJECT_HELP_TEXT,
    )
    project_space_group.add_argument(
        '--space',
        required=False,
        help=_SPACE_HELP_TEXT,
    )

  @staticmethod
  def ValidSharedTemplateRevisionName(name):
    """Validates the shared template revision name."""
    pattern = re.compile(
        r'^projects/[^/]+/locations/[^/]+/spaces/[^/]+/sharedTemplates/[^/]+/revisions(/[^/]+)?$'
    )
    return pattern.match(name) is not None

  def Run(self, args):
    """Run the describe command."""
    client = apis.SharedTemplateRevisionsClient(
        release_track=base.ReleaseTrack.ALPHA
    )

    # If the full resource name is provided, use it directly.
    if self.ValidSharedTemplateRevisionName(args.revision):
      if (
          args.IsSpecified('shared_template')
          or args.IsSpecified('location')
          or args.IsSpecified('project')
          or args.IsSpecified('space')
          or args.google_catalog
      ):
        raise exceptions.ConflictingArgumentsException(
            '--shared-template, --location, --project, --space, and'
            ' --google-catalog cannot be used when a full resource name is'
            ' provided.'
        )
      return client.Describe(name=args.revision)

    # Otherwise, build the resource name from flags.
    if not args.IsSpecified('location'):
      raise SharedTemplateRevisionResourceError(
          f'{_BASE_ERROR_MESSAGE}\n{_LOCATION_NOT_SPECIFIED_MESSAGE}'
      )
    location_id = args.location

    if args.google_catalog:
      project_id = api_lib_utils.GetGoogleCatalogProjectId()
      space_id = 'googlespace'
    else:
      project_id = args.project or properties.VALUES.core.project.Get()
      if not project_id:
        raise SharedTemplateRevisionResourceError(
            f'{_BASE_ERROR_MESSAGE}\n{_PROJECT_NOT_SPECIFIED_MESSAGE}'
        )
      if not args.IsSpecified('space'):
        raise SharedTemplateRevisionResourceError(
            f'{_BASE_ERROR_MESSAGE}\n{_SPACE_NOT_SPECIFIED_MESSAGE}'
        )
      space_id = args.space

    if not args.IsSpecified('shared_template'):
      raise SharedTemplateRevisionResourceError(
          f'{_BASE_ERROR_MESSAGE}\n{_SHARED_TEMPLATE_NOT_SPECIFIED_MESSAGE}'
      )
    shared_template_id = args.shared_template

    # Construct the resource name:
    # projects/{p}/locations/{l}/spaces/{s}/sharedTemplates/{st}/revisions/{str}
    shared_template_revision_name = (
        'projects/{}/locations/{}/spaces/{}/sharedTemplates/{}/revisions/{}'
        .format(
            project_id, location_id, space_id, shared_template_id, args.revision
        )
    )

    return client.Describe(name=shared_template_revision_name)


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.UniverseCompatible
class DescribeGa(base.DescribeCommand):
  """Describe a shared template revision."""

  detailed_help = _DETAILED_HELP_GA

  @staticmethod
  def Args(parser):
    # Positional argument for revision
    parser.add_argument(
        'revision',
        metavar='REVISION',
        help=_REVISION_HELP_TEXT,
    )
    # Shared template flag
    parser.add_argument(
        '--shared-template',
        required=False,
        help=_SHARED_TEMPLATE_HELP_TEXT,
    )
    # Location flag
    parser.add_argument(
        '--location',
        required=False,
        help=_LOCATION_HELP_TEXT,
    )

    # Argument group for --project/--space or --google-catalog
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '--google-catalog',
        action='store_true',
        help=_GOOGLE_CATALOG_HELP_TEXT,
    )
    # Nested group for --project and --space
    project_space_group = group.add_argument_group(
        'Specify --project and --space for custom shared templates.'
    )
    project_space_group.add_argument(
        '--project',
        required=False,
        help=_PROJECT_HELP_TEXT,
    )
    project_space_group.add_argument(
        '--space',
        required=False,
        help=_SPACE_HELP_TEXT,
    )

  @staticmethod
  def ValidSharedTemplateRevisionName(name):
    """Validates the shared template revision name."""
    pattern = re.compile(
        r'^projects/[^/]+/locations/[^/]+/spaces/[^/]+/sharedTemplates/[^/]+/revisions(/[^/]+)?$'
    )
    return pattern.match(name) is not None

  def Run(self, args):
    """Run the describe command."""
    client = apis.SharedTemplateRevisionsClient(
        release_track=base.ReleaseTrack.GA
    )

    # If the full resource name is provided, use it directly.
    if self.ValidSharedTemplateRevisionName(args.revision):
      if (
          args.IsSpecified('shared_template')
          or args.IsSpecified('location')
          or args.IsSpecified('project')
          or args.IsSpecified('space')
          or args.google_catalog
      ):
        raise exceptions.ConflictingArgumentsException(
            '--shared-template, --location, --project, --space, and'
            ' --google-catalog cannot be used when a full resource name is'
            ' provided.'
        )
      return client.Describe(name=args.revision)

    # Otherwise, build the resource name from flags.
    if not args.IsSpecified('location'):
      raise SharedTemplateRevisionResourceError(
          f'{_BASE_ERROR_MESSAGE}\n{_LOCATION_NOT_SPECIFIED_MESSAGE}'
      )
    location_id = args.location

    if args.google_catalog:
      project_id = api_lib_utils.GetGoogleCatalogProjectId()
      space_id = 'googlespace'
    else:
      project_id = args.project or properties.VALUES.core.project.Get()
      if not project_id:
        raise SharedTemplateRevisionResourceError(
            f'{_BASE_ERROR_MESSAGE}\n{_PROJECT_NOT_SPECIFIED_MESSAGE}'
        )
      if not args.IsSpecified('space'):
        raise SharedTemplateRevisionResourceError(
            f'{_BASE_ERROR_MESSAGE}\n{_SPACE_NOT_SPECIFIED_MESSAGE}'
        )
      space_id = args.space

    if not args.IsSpecified('shared_template'):
      raise SharedTemplateRevisionResourceError(
          f'{_BASE_ERROR_MESSAGE}\n{_SHARED_TEMPLATE_NOT_SPECIFIED_MESSAGE}'
      )
    shared_template_id = args.shared_template

    # Construct the resource name:
    # projects/{p}/locations/{l}/spaces/{s}/sharedTemplates/{st}/revisions/{str}
    shared_template_revision_name = (
        'projects/{}/locations/{}/spaces/{}/sharedTemplates/{}/revisions/{}'
        .format(
            project_id, location_id, space_id, shared_template_id, args.revision
        )
    )

    return client.Describe(name=shared_template_revision_name)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# NOTE: This file is autogenerated and should not be edited by hand.
# AUTOGEN_CLI_VERSION: HEAD
"""File to add optional custom code to extend __init__.py."""
from googlecloudsdk.calliope import base


class AuthorizationsAlpha(base.Group):
  """Optional no-auto-generated code for ALPHA."""


class AuthorizationsBeta(base.Group):
  """Optional no-auto-generated code for BETA."""


class AuthorizationsGa(base.Group):
  """Optional no-auto-generated code for GA."""



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""service-management configs list command."""


from apitools.base.py import list_pager

from googlecloudsdk.api_lib.endpoints import services_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.endpoints import arg_parsers


class List(base.ListCommand):
  """Lists the configurations for a given service.

  This command lists all the configurations for a given service by ID.

  To get more detailed information about a specific configuration, run:

    $ {parent_command} describe

  ## EXAMPLES

  To list the configurations for a service named `my-service`, run:

    $ {command} --service=my-service
  """

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    parser.add_argument(
        '--service',
        required=True,
        help='The name of service for which to list existing configurations.')
    parser.display_info.AddFormat("""
          table(
            id:label=CONFIG_ID,
            name:label=SERVICE_NAME
          )
        """)

  def Run(self, args):
    """Run 'service-management configs list'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      The response from the List API call.
    """
    messages = services_util.GetMessagesModule()
    client = services_util.GetClientInstance()

    service = arg_parsers.GetServiceNameFromArg(args.service)

    request = messages.ServicemanagementServicesConfigsListRequest(
        serviceName=service)

    return list_pager.YieldFromList(
        client.services_configs,
        request,
        limit=args.limit,
        batch_size_attribute='pageSize',
        batch_size=args.page_size,
        field='serviceConfigs')



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Enable Identity-Aware Proxy."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iap import util as iap_util


@base.ReleaseTracks(
        base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
    )
@base.DefaultUniverseOnly
class Enable(base.Command):
  """Enable Cloud Identity-Aware Proxy (Cloud IAP) on an IAP resource.

  This command enables Cloud Identity-Aware Proxy on an IAP resource. OAuth 2.0
  credentials must be set, or must have been previously set, to enable IAP.
  """
  detailed_help = {
      'EXAMPLES':
          """\
          To enable IAP on an App Engine application, run:

            $ {command} --resource-type=app-engine
                --oauth2-client-id=CLIENT_ID --oauth2-client-secret=SECRET

          To enable IAP on a global backend service, run:

            $ {command} --resource-type=backend-services
                --oauth2-client-id=CLIENT_ID --oauth2-client-secret=SECRET
                --service=SERVICE_ID

          To enable IAP on a region backend service, run:

            $ {command} --resource-type=backend-services
                --oauth2-client-id=CLIENT_ID --oauth2-client-secret=SECRET
                --service=SERVICE_ID --region=REGION
  """,
  }

  @classmethod
  def Args(cls, parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
          to capture some information, but behaves like an ArgumentParser.
    """
    iap_util.AddIapResourceArgs(parser)
    iap_util.AddOauthClientArgs(parser)
    base.URI_FLAG.RemoveFromParser(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The specified function with its description and configured filter.
    """
    iap_ref = iap_util.ParseIapResource(self.ReleaseTrack(), args)
    return iap_ref.Enable(args.oauth2_client_id, args.oauth2_client_secret)




# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""List command for gcloud debug logpoints command group."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.data_catalog import flags
from googlecloudsdk.command_lib.data_catalog.taxonomies import taxonomies
from googlecloudsdk.command_lib.data_catalog.taxonomies import util
from googlecloudsdk.core import properties


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Import(base.Command):
  """Export a file with serialized taxonomies to a certain project."""

  detailed_help = {
      'DESCRIPTION': """
          Export a file with serialized taxonomies to a certain project.
      """,
      'EXAMPLES': """
          To parse and import the taxonomies contained in '/tmp/taxonomies.json'
          to your project within location LOCATION:

            $ {command} "/tmp/taxonomies.json" --location="LOCATION"
      """,
  }

  @staticmethod
  def Args(parser):
    flags.AddLocationResourceArg(parser, 'Location to import taxonomies to.')
    parser.add_argument(
        'taxonomies',
        type=arg_parsers.YAMLFileContents(),
        metavar='TAXONOMIES',
        help="""\
            File containing serialized taxonomy.
        """)

  def Run(self, args):
    """Run the import command."""
    version_label = flags.GetVersionLabel(self.ReleaseTrack())
    args.project_val = properties.VALUES.core.project.Get()
    args.taxonomies = util.ProcessTaxonomiesFromYAML(args.taxonomies,
                                                     version_label)
    return taxonomies.Import(args, version_label)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud monitoring policies conditions delete` command."""


from googlecloudsdk.api_lib.monitoring import policies
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.monitoring import resource_args
from googlecloudsdk.command_lib.monitoring import util
from googlecloudsdk.core import log


class Delete(base.CreateCommand):
  """Delete a condition in an alerting policy.

  Delete a condition in an alerting policy. If the specified condition does
  not exist, this command will fail with an error. This will not delete
  the policy if no conditions exist.
  """

  @staticmethod
  def Args(parser):
    condition_arg = resource_args.CreateConditionResourceArg('delete')
    resource_args.AddResourceArgs(parser, [condition_arg])

  def Run(self, args):
    client = policies.AlertPolicyClient()
    condition_ref = args.CONCEPTS.condition.Parse()
    policy_ref = condition_ref.Parent()
    policy = client.Get(policy_ref)

    policy = util.RemoveConditionFromPolicy(condition_ref.RelativeName(),
                                            policy)

    response = client.Update(policy_ref, policy)
    log.DeletedResource(condition_ref.RelativeName(), 'Condition')
    return response



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to show fleet scopes in a project."""


from googlecloudsdk.api_lib.container.fleet import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet import util
from googlecloudsdk.core import properties


class List(base.ListCommand):
  """List fleet scopes in a project permitted to be viewed by the caller.

  This command can fail for the following reasons:
  * The project specified does not exist.
  * The user does not have access to the project specified.

  ## EXAMPLES

  The following command lists scopes in the active project:

    $ {command}

  The following command lists scopes in project `PROJECT_ID`:

    $ {command} --project=PROJECT_ID
  """

  @staticmethod
  def Args(parser):
    # Table formatting
    parser.display_info.AddFormat(util.SC_LIST_FORMAT)

  def Run(self, args):
    project = args.project
    if project is None:
      project = properties.VALUES.core.project.Get()
    fleetclient = client.FleetClient(release_track=self.ReleaseTrack())
    return fleetclient.ListPermittedScopes(project)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""List operation command."""


import json

from apitools.base.py import list_pager

from googlecloudsdk.api_lib.dataproc import constants
from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.api_lib.dataproc import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataproc import flags
from googlecloudsdk.core import properties


STATE_MATCHER_MAP = {'active': 'ACTIVE', 'inactive': 'NON_ACTIVE'}
STATE_MATCHER_FILTER = 'operation_state_matcher'
CLUSTER_NAME_FILTER = 'cluster_name'
PROJECT_FILTER = 'project_id'


class List(base.ListCommand):
  """View the list of all operations.

  View a list of operations in a project. An optional filter can be used to
  constrain the operations returned. Filters are case-sensitive and have the
  following syntax:

    field = value [AND [field = value]] ...

  where `field` is either of `status.state` or `labels.[KEY]`, where `[KEY]` is
  a label key. `value` can be ```*``` to match all values. `status.state` is
  one of: `PENDING`, `ACTIVE`, `DONE`. Only the logical `AND` operator is
  supported; space-separated items are treated as having an implicit `AND`
  operator.

  ## EXAMPLES

  To see the list of all operations in Dataproc's 'us-central1' region, run:

    $ {command} --region='us-central1'

  To see the list of all create cluster operations in Dataproc's 'global'
  region, run:

    $ {command} --region='global' --filter='operationType = CREATE'

  To see the list of all active operations in a cluster named 'mycluster'
  located in Dataproc's 'global' region, run:

    $ {command} --region='global' --filter='status.state = RUNNING AND
      clusterName = mycluster'

  To see a list of all pending operations with the label `env=staging` on
  cluster `mycluster` located in Dataproc's 'us-central1' region, run:

    $ {command} --region='us-central1' --filter='status.state = PENDING
      AND labels.env = staging AND clusterName = mycluster'
  """

  @staticmethod
  def Args(parser):
    base.URI_FLAG.RemoveFromParser(parser)
    base.PAGE_SIZE_FLAG.SetDefault(parser, constants.DEFAULT_PAGE_SIZE)

    flags.AddRegionFlag(parser)

    parser.add_argument(
        '--cluster',
        help=('Restrict to the operations of this Dataproc cluster. This flag '
              'is ignored when --filter is specified. The equivalent term in '
              'a --filter expression is: `clusterName = myclustername`'))

    parser.add_argument(
        '--state-filter',
        choices=sorted(STATE_MATCHER_MAP.keys()),
        help=('Filter by cluster state. This flag is ignored when --filter '
              'is specified. The equivalent term in a --filter expression is: '
              '`status.state = ACTIVE`'))

    flags.AddListOperationsFormat(parser)

  def Run(self, args):
    dataproc = dp.Dataproc(self.ReleaseTrack())

    region_callback = util.ResolveRegion
    # Parse Operations endpoint.
    project_callback = properties.VALUES.core.project.GetOrFail
    operation_list_ref = dataproc.resources.Parse(
        None,
        params={'regionId': region_callback, 'projectId': project_callback},
        collection='dataproc.projects.regions.operations_list')

    filter_dict = dict()
    if args.state_filter:
      filter_dict[STATE_MATCHER_FILTER] = STATE_MATCHER_MAP[args.state_filter]
    if args.cluster:
      filter_dict[CLUSTER_NAME_FILTER] = args.cluster

    op_filter = None
    if args.filter:
      # Prefer new filter argument if present
      op_filter = args.filter
      # Explicitly null out args.filter if present because by default
      # args.filter also acts as a postfilter to the things coming back from the
      # backend
      args.filter = None
    else:
      op_filter = json.dumps(filter_dict)

    request = dataproc.messages.DataprocProjectsRegionsOperationsListRequest(
        name=operation_list_ref.RelativeName(), filter=op_filter)

    return list_pager.YieldFromList(
        dataproc.client.projects_regions_operations,
        request,
        limit=args.limit, field='operations',
        batch_size=args.page_size,
        batch_size_attribute='pageSize')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to delete the specified channel connection."""


from googlecloudsdk.api_lib.eventarc import channel_connections
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.eventarc import flags
from googlecloudsdk.core.console import console_io

_DETAILED_HELP = {
    'DESCRIPTION':
        '{description}',
    'EXAMPLES':
        """\
        To delete the channel connection ``my-channel-connection'' in location ``us-central1'', run:

          $ {command} my-channel-connection --location=us-central1
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Delete(base.DeleteCommand):
  """Delete an Eventarc channel connection."""

  detailed_help = _DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    flags.AddChannelConnectionResourceArg(parser,
                                          'Channel connection to delete.')
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    client = channel_connections.ChannelConnectionClientV1()
    channel_connection_ref = args.CONCEPTS.channel_connection.Parse()

    location_name = channel_connection_ref.Parent().Name()
    console_io.PromptContinue(
        message=('The following channel connection will be deleted.\n'
                 '[{name}] in location [{location}]'.format(
                     name=channel_connection_ref.Name(),
                     location=location_name)),
        throw_if_unattended=True,
        cancel_on_no=True)
    operation = client.Delete(channel_connection_ref)

    if args.async_:
      return operation
    return client.WaitFor(operation, 'Deleting', channel_connection_ref)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Submit a SparkR batch job."""


from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataproc.batches import batch_submitter
from googlecloudsdk.command_lib.dataproc.batches import sparkr_batch_factory


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class SparkR(base.Command):
  """Submit a Spark R batch job."""
  detailed_help = {
      'EXAMPLES':
          """\
          To submit a Spark R batch job running "my-main-r.r" script and upload it to "gs://my-bucket", run:

            $ {command} my-main-r.r --deps-bucket=gs://my-bucket --region='us-central1' -- ARG1 ARG2
          """
  }

  @staticmethod
  def Args(parser):
    sparkr_batch_factory.AddArguments(parser)

  def Run(self, args):
    dataproc = dp.Dataproc(base.ReleaseTrack.GA)
    sparkr_batch = sparkr_batch_factory.SparkRBatchFactory(
        dataproc).UploadLocalFilesAndGetMessage(args)

    return batch_submitter.Submit(sparkr_batch, dataproc, args)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update fleet information."""


from googlecloudsdk.api_lib.container.fleet import client
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet import resources
from googlecloudsdk.command_lib.util.apis import arg_utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Update(base.UpdateCommand):
  """Update a fleet namespace.

  This command can fail for the following reasons:
  * The project specified does not exist.
  * The fleet namespace does not exist in the project.
  * The caller does not have permission to access the project or namespace.

  ## EXAMPLES

  To update the namespace `NAMESPACE` in the active project:

    $ {command} NAMESPACE

  To update the namespace `NAMESPACE` in project `PROJECT_ID`:

    $ {command} NAMESPACE --project=PROJECT_ID
  """

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        'NAME', type=str, help='Name of the namespace to be updated.')
    resources.AddScopeResourceArg(
        parser,
        '--scope',
        util.VERSION_MAP[cls.ReleaseTrack()],
        scope_help='Name of the fleet scope to create the fleet namespace in.',
    )

  def Run(self, args):
    mask = []
    for flag in ['scope']:
      if args.IsKnownAndSpecified(flag):
        mask.append(flag)
    scope = None
    scope_arg = args.CONCEPTS.scope.Parse()
    if scope_arg is not None:
      scope = scope_arg.RelativeName()
    project = arg_utils.GetFromNamespace(args, '--project', use_defaults=True)
    fleetclient = client.FleetClient(release_track=self.ReleaseTrack())
    return fleetclient.UpdateNamespace(
        args.NAME, scope, project, mask=','.join(mask)
    )



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List deployment groups command."""


from googlecloudsdk.api_lib.network_security.intercept_deployment_groups import api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.network_security.intercept import deployment_group_flags

DETAILED_HELP = {
    'DESCRIPTION': """
          List intercept deployment groups.

          For more examples, refer to the EXAMPLES section below.

        """,
    'EXAMPLES': """
            To list intercept deployment groups in project ID `my-project`, run:

            $ {command} --location=global --project=my-project

            OR

            $ {command} --location=global

        """,
}

_FORMAT = """\
table(
    name.scope("interceptDeploymentGroups"):label=ID,
    name.scope("locations").segment(0):label=LOCATION,
    state
)
"""


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class List(base.ListCommand):
  """List Intercept Deployment Groups."""

  @classmethod
  def Args(cls, parser):
    parser.display_info.AddFormat(_FORMAT)
    parser.display_info.AddUriFunc(
        deployment_group_flags.MakeGetUriFunc(cls.ReleaseTrack())
    )
    deployment_group_flags.AddLocationResourceArg(
        parser, help_text='The location for a list operation'
    )

  def Run(self, args):
    client = api.Client(self.ReleaseTrack())

    parent_ref = args.CONCEPTS.location.Parse()

    return client.ListDeploymentGroups(parent_ref.RelativeName(),
                                       page_size=args.page_size)


List.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to delete one or more Dataproc Metastore services."""


import json

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.metastore import services_util as services_api_util
from googlecloudsdk.api_lib.metastore import util as api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.metastore import delete_util
from googlecloudsdk.command_lib.metastore import resource_args
from googlecloudsdk.command_lib.metastore import util as command_util
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io

DETAILED_HELP = {
    'EXAMPLES':
        """\
          To delete a Dataproc Metastore service with the name
          `my-metastore-service` in location `us-central1`, run:

          $ {command} my-metastore-service --location=us-central1

          To delete multiple Dataproc Metastore services with the name
          `service-1` and `service-2` in the same location
          `us-central1`, run:

          $ {command} service-1 service-2 --location=us-central1
        """
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Delete(base.DeleteCommand):
  """Delete one or more Dataproc Metastore services.

  If run asynchronously with `--async`, exits after printing
  one or more operation names that can be used to poll the status of the
  deletion(s) via:

    {top_command} metastore operations describe
  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    resource_args.AddServiceResourceArg(
        parser, 'to delete', plural=True, required=True, positional=True)
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    env_refs = args.CONCEPTS.services.Parse()
    console_io.PromptContinue(
        message=command_util.ConstructList('Deleting the following services:', [
            '[{}] in [{}]'.format(env_ref.servicesId, env_ref.locationsId)
            for env_ref in env_refs
        ]),
        cancel_on_no=True,
        cancel_string='Deletion aborted by user.',
        throw_if_unattended=True)

    waiter = delete_util.ServiceDeletionWaiter(
        release_track=self.ReleaseTrack())
    encountered_errors = False
    for env_ref in env_refs:
      operation = None
      failed = None
      try:
        operation = services_api_util.Delete(
            env_ref.RelativeName(), release_track=self.ReleaseTrack())
      except apitools_exceptions.HttpError as e:
        failed = json.loads(e.content)['error']['message']
        encountered_errors = True
      else:
        details = 'with operation [{0}]'.format(operation.name)
        waiter.AddPendingDelete(
            service_name=env_ref.RelativeName(), operation=operation)
      finally:
        log.DeletedResource(
            env_ref.RelativeName(),
            kind='service',
            is_async=True,
            details=None if encountered_errors else
            'with operation [{0}]'.format(operation.name),
            failed=failed)

    if not args.async_:
      encountered_errors = waiter.Wait() or encountered_errors
    if encountered_errors:
      raise api_util.ServiceDeleteError(
          'Some requested deletions did not succeed.')



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""'types describe' command."""


from googlecloudsdk.api_lib.deployment_manager import dm_base
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
@dm_base.UseDmApi(dm_base.DmApiVersion.V2BETA)
class Describe(base.DescribeCommand, dm_base.DmCommand):
  """Describe a type."""

  detailed_help = {
      'EXAMPLES': """\
          To display information about a type provider type

            $ {command} NAME --provider=PROVIDER

          If you want to see information for a composite type you can use

            $ {command} NAME --provider=composite --format="yaml[json-decode] (composite_type)"
          """,
  }

  @staticmethod
  def Args(parser):
    """Called by calliope to gather arguments for this command.

    Args:
      parser: argparse parser for specifying command line arguments
    """
    parser.add_argument('name', help='Type name.')
    parser.add_argument('--provider',
                        help='Type provider name or its self-link.',
                        required=True)
    parser.display_info.AddFormat('yaml[json-decode](type_info)')

  def Run(self, args):
    """Runs 'types describe'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      The requested TypeInfo.

    Raises:
      HttpException: An http error response was received while executing the api
          request.
      InvalidArgumentException: The requested type provider type could not
          be found.
    """
    type_provider_ref = self.resources.Parse(
        args.provider,
        params={'project': properties.VALUES.core.project.GetOrFail},
        collection='deploymentmanager.typeProviders')
    request = self.messages.DeploymentmanagerTypeProvidersGetTypeRequest(
        project=type_provider_ref.project,
        type=args.name,
        typeProvider=type_provider_ref.typeProvider)
    type_message = self.client.typeProviders.GetType(request)

    composite_type_message = 'This is not a composite type.'
    if type_provider_ref.typeProvider == 'composite':
      composite_request = (self.messages.
                           DeploymentmanagerCompositeTypesGetRequest(
                               project=type_provider_ref.project,
                               compositeType=args.name))
      composite_type_message = self.client.compositeTypes.Get(composite_request)

    log.status.Print('You can reference this type in Deployment Manager with '
                     '[{0}/{1}:{2}]'.format(type_provider_ref.project,
                                            type_provider_ref.typeProvider,
                                            args.name))
    return {'type_info': type_message,
            'composite_type': composite_type_message}



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create a release."""


import datetime
import os.path

from googlecloudsdk.api_lib.clouddeploy import client_util
from googlecloudsdk.api_lib.clouddeploy import release
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as c_exceptions
from googlecloudsdk.command_lib.deploy import delivery_pipeline_util
from googlecloudsdk.command_lib.deploy import deploy_policy_util
from googlecloudsdk.command_lib.deploy import deploy_util
from googlecloudsdk.command_lib.deploy import flags
from googlecloudsdk.command_lib.deploy import promote_util
from googlecloudsdk.command_lib.deploy import release_util
from googlecloudsdk.command_lib.deploy import resource_args
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import resources
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import times


_DETAILED_HELP = {
    'DESCRIPTION': '{description}',
    'EXAMPLES': """ \
    To create a release with source located at storage URL `gs://bucket/object.zip`
    and the first rollout in the first target of the promotion sequence:

       $ {command} my-release --source=`gs://bucket/object.zip` --delivery-pipeline=my-pipeline --region=us-central1

    To create a release with source located at current directory
    and deploy a rollout to target prod :

      $ {command} my-release --delivery-pipeline=my-pipeline --region=us-central1 --to-target=prod

    The following command creates a release without a `skaffold.yaml` as input, and generates one
    for you:

      $ {command} my-release --delivery-pipeline=my-pipeline --region=us-central1 --from-k8s-manifest=path/to/kubernetes/k8.yaml

    The current UTC date and time on the machine running the gcloud command can
    also be included in the release name by adding $DATE and $TIME parameters:

      $ {command} 'my-release-$DATE-$TIME' --delivery-pipeline=my-pipeline --region=us-central1

    If the current UTC date and time is set to 2021-12-21 12:02, then the created release
    will have its name set as my-release-20211221-1202.

    When using these parameters, please be sure to wrap the release name in single quotes
    or else the template parameters will be overridden by environment variables.
    """,
}
_RELEASE = 'release'
_MAINTENANCE_WARNING_DAYS = 28


def _CommonArgs(parser):
  """Register flags for this command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in order to
      capture some information, but behaves like an ArgumentParser.
  """
  resource_args.AddReleaseResourceArg(parser, positional=True, required=True)
  flags.AddGcsSourceStagingDirFlag(parser)
  flags.AddImagesGroup(parser)
  flags.AddIgnoreFileFlag(parser)
  flags.AddToTargetFlag(parser)
  flags.AddDescription(parser, 'Description of the release.')
  flags.AddAnnotationsFlag(parser, _RELEASE)
  flags.AddLabelsFlag(parser, _RELEASE)
  flags.AddDockerVersion(parser)
  flags.AddHelmVersion(parser)
  flags.AddKptVersion(parser)
  flags.AddKubectlVersion(parser)
  flags.AddKustomizeVersion(parser)
  flags.AddSkaffoldVersion(parser)
  flags.AddSkaffoldOrNativeSources(parser)
  flags.AddInitialRolloutGroup(parser)
  flags.AddDeployParametersFlag(parser)
  flags.AddOverrideDeployPolicies(parser)


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.DefaultUniverseOnly
class Create(base.CreateCommand):
  """Creates a new release, delivery pipeline qualified."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    _CommonArgs(parser)

  def _CheckIfNearMaintenance(self, release_obj):
    """Checks to see if a release is close to the maintenance window."""

    def _ParseDt(dt):
      """Parses the maintenance dt, returning a datetime or None."""
      if dt is None:
        return None
      try:
        return times.ParseDateTime(dt)
      except (times.DateTimeSyntaxError, times.DateTimeValueError):
        return None

    release_condition = release_obj.condition
    if release_condition is None:
      return

    has_tool_versions = (
        release_condition.dockerVersionSupportedCondition
        or release_condition.helmVersionSupportedCondition
        or release_condition.kptVersionSupportedCondition
        or release_condition.kubectlVersionSupportedCondition
        or release_condition.kustomizeVersionSupportedCondition
        or release_condition.skaffoldVersionSupportedCondition
    )
    if not has_tool_versions:
      if release_condition.skaffoldSupportedCondition is None:
        return
      maintenance_dt = _ParseDt(
          release_condition.skaffoldSupportedCondition.maintenanceModeTime
      )
      # It is possible to have no maintenance mode time.
      # Like `skaffold_preview` for example.
      if (
          maintenance_dt is not None
          and maintenance_dt - times.Now()
          <= datetime.timedelta(days=_MAINTENANCE_WARNING_DAYS)
      ):
        log.status.Print(
            "WARNING: This release's Skaffold version will be"
            ' in maintenance mode beginning on {date}.'
            " After that you won't be able to create releases"
            ' using this version of Skaffold.\n'
            'https://cloud.google.com/deploy/docs/using-skaffold'
            '/select-skaffold#skaffold_version_deprecation'
            '_and_maintenance_policy'.format(
                date=maintenance_dt.strftime('%Y-%m-%d')
            )
        )
      return

    # use an array to have deterministic ordering.
    tools_supported_condition_to_process = [
        (
            release_util.Tools.DOCKER,
            release_condition.dockerVersionSupportedCondition,
        ),
        (
            release_util.Tools.HELM,
            release_condition.helmVersionSupportedCondition,
        ),
        (
            release_util.Tools.KPT,
            release_condition.kptVersionSupportedCondition,
        ),
        (
            release_util.Tools.KUBECTL,
            release_condition.kubectlVersionSupportedCondition,
        ),
        (
            release_util.Tools.KUSTOMIZE,
            release_condition.kustomizeVersionSupportedCondition,
        ),
        (
            release_util.Tools.SKAFFOLD,
            release_condition.skaffoldVersionSupportedCondition,
        ),
    ]
    tools_almost_in_maintenance = []
    for tool, condition in tools_supported_condition_to_process:
      if not condition:
        continue
      maintenance_dt = _ParseDt(condition.maintenanceModeTime)
      if (
          maintenance_dt is not None
          and maintenance_dt - times.Now()
          <= datetime.timedelta(days=_MAINTENANCE_WARNING_DAYS)
      ):
        tools_almost_in_maintenance.append(tool)
    if tools_almost_in_maintenance:
      joined = ', '.join([tool.value for tool in tools_almost_in_maintenance])
      log.status.Print(
          f'WARNING: The versions used for tools: [{joined}] will be in'
          " maintenance mode soon. After that you won't be able to create"
          ' releases using these versions of the tools.\n'
          'https://cloud.google.com/deploy/docs/select-tool-version'
      )

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: All the arguments that were provided to this command invocation.

    Returns:
      The release and rollout created.
    """
    if args.disable_initial_rollout and args.to_target:
      raise c_exceptions.ConflictingArgumentsException(
          '--disable-initial-rollout', '--to-target'
      )

    args.CONCEPTS.parsed_args.release = release_util.RenderPattern(
        args.CONCEPTS.parsed_args.release
    )
    release_ref = args.CONCEPTS.release.Parse()
    pipeline_obj = delivery_pipeline_util.GetPipeline(
        release_ref.Parent().RelativeName()
    )
    failed_activity_msg = 'Cannot create release {}.'.format(
        release_ref.RelativeName()
    )
    delivery_pipeline_util.ThrowIfPipelineSuspended(
        pipeline_obj, failed_activity_msg
    )
    actual_source, native_source_specified = release_util.GetActualSource(args)
    # Only when the skaffold file is an absolute path needs to be handled
    # here.
    if args.skaffold_file and os.path.isabs(args.skaffold_file):
      # If source is not specified, use current directory.
      # The source is only used for the file path check, it's not used in the
      # CreateReleaseConfig.
      source_description = (
          'current working directory' if args.source == '.' else 'source'
      )
      if not files.IsDirAncestorOf(actual_source, args.skaffold_file):
        raise core_exceptions.Error(
            f'The skaffold file {args.skaffold_file} could not be found in the'
            f' {source_description}. Please enter a valid Skaffold file path.'
        )
      args.skaffold_file = os.path.relpath(
          os.path.abspath(args.skaffold_file), os.path.abspath(actual_source)
      )
    config_check_mode = release_util.GetConfigCheckModeFromArgs(args)
    native_config_used = release_util.VerifyConfigs(
        actual_source, config_check_mode, args.skaffold_file, args.config_file
    )

    client = release.ReleaseClient()
    # Create the release create request.
    release_config = release_util.CreateReleaseConfig(
        source=actual_source,
        gcs_source_staging_dir=args.gcs_source_staging_dir,
        ignore_file=args.ignore_file,
        images=args.images,
        build_artifacts=args.build_artifacts,
        description=args.description,
        docker_version=args.docker_version,
        helm_version=args.helm_version,
        kpt_version=args.kpt_version,
        kubectl_version=args.kubectl_version,
        kustomize_version=args.kustomize_version,
        skaffold_version=args.skaffold_version,
        skaffold_file=args.skaffold_file,
        location=release_ref.AsDict()['locationsId'],
        pipeline_uuid=pipeline_obj.uid,
        from_k8s_manifest=args.from_k8s_manifest,
        from_run_manifest=args.from_run_manifest,
        pipeline_obj=pipeline_obj,
        deploy_parameters=args.deploy_parameters,
        config_file=args.config_file,
        native_config_used=native_config_used,
        native_source_specified=native_source_specified,
    )

    deploy_util.SetMetadata(
        client.messages,
        release_config,
        deploy_util.ResourceType.RELEASE,
        args.annotations,
        args.labels,
    )
    operation = client.Create(release_ref, release_config)
    operation_ref = resources.REGISTRY.ParseRelativeName(
        operation.name, collection='clouddeploy.projects.locations.operations'
    )
    client_util.OperationsClient().WaitForOperation(operation, operation_ref)

    log.status.Print(
        'Created Cloud Deploy release {}.'.format(release_ref.Name())
    )

    release_obj = release.ReleaseClient().Get(release_ref.RelativeName())
    self._CheckIfNearMaintenance(release_obj)
    if args.disable_initial_rollout:
      return release_obj
    # On the command line deploy policy IDs are provided, but for the
    # CreateRollout API we need to provide the full resource name.
    pipeline_ref = release_ref.Parent()
    policies = deploy_policy_util.CreateDeployPolicyNamesFromIDs(
        pipeline_ref, args.override_deploy_policies
    )
    rollout_resource = promote_util.Promote(
        release_ref,
        release_obj,
        args.to_target,
        is_create=True,
        labels=args.initial_rollout_labels,
        annotations=args.initial_rollout_annotations,
        starting_phase_id=args.initial_rollout_phase_id,
        override_deploy_policies=policies,
    )

    return release_obj, rollout_resource



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a catalog template revision."""


from googlecloudsdk.api_lib.design_center import utils
from googlecloudsdk.api_lib.util import messages as messages_util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.design_center import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import yaml

_DETAILED_HELP = {
    'DESCRIPTION': 'Create a new catalog template revision.',
    'EXAMPLES': """ \
        To create a new catalog template revision named `my-revision` for the template `my-template`, within the catalog `my-catalog` and space `my-space`, in location `us-central1` and project `my-project`, using a Developer Connect repository as the source, run:

          $ {command} my-revision --project=my-project --location=us-central1 \\
            --space=my-space \\
            --catalog=my-catalog \\
            --template=my-template \\
            --description="My test revision description" \\
            --developer-connect-repo=projects/my-project/locations/us-central1/connections/my-connection/gitRepositoryLinks/my-repo \\
            --developer-connect-repo-ref=refs/tags/v1.0.0 \\
            --developer-connect-repo-dir=modules/my-product

          Or run using the full resource name:

            $ {command} projects/my-project/locations/us-central1/spaces/my-space/catalogs/my-catalog/templates/my-template/revisions/my-revision \\
            --description="My test revision description" \\
            --developer-connect-repo=projects/my-project/locations/us-central1/connections/my-connection/gitRepositoryLinks/my-repo \\
            --developer-connect-repo-ref=refs/tags/v1.0.0 \\
            --developer-connect-repo-dir=modules/my-product

          To create a revision using a metadata file from a local path, run:

            $ {command} my-revision --project=my-project --location=us-central1 \\
            --space=my-space \\
            --catalog=my-catalog \\
            --template=my-template \\
            --developer-connect-repo=projects/my-project/locations/us-central1/connections/my-connection/gitRepositoryLinks/my-repo \\
            --developer-connect-repo-ref=refs/tags/v1.0.0 \\
            --developer-connect-repo-dir=modules/my-product \\
            --metadata=/path/to/metadata.yaml

          To create a revision using a Git repository as the source, run:

            $ {command} my-revision --project=my-project --location=us-central1 \\
            --space=my-space \\
            --catalog=my-catalog \\
            --template=my-template \\
            --git-source-repo=GoogleCloudPlatform/terraform-google-cloud-run \\
            --git-source-ref-tag=v1.0.0 \\
            --git-source-dir=modules/my-product

          To create a revision using a Google Cloud Storage URI as the source, run:

            $ {command} my-revision --project=my-project --location=us-central1 \\
            --space=my-space \\
            --catalog=my-catalog \\
            --template=my-template \\
            --gcs-source-uri=gs://my-bucket/my-template

          To create a revision using an OCI repository as the source, run:

            $ {command} my-revision --project=my-project --location=us-central1 \\
            --space=my-space \\
            --catalog=my-catalog \\
            --template=my-template \\
            --oci-repo-uri=oci://us-west1-docker.pkg.dev/my-project/my-repo/my-chart \\
            --oci-repo-version=1.0.0

          To create a revision using an Application Template Revision as the source, run:

            $ {command} my-revision --project=my-project --location=us-central1 \\
            --space=my-space \\
            --catalog=my-catalog \\
            --template=my-template \\
            --application-template-revision-source=projects/my-project/locations/us-central1/spaces/my-space/applicationTemplates/my-app-template/revisions/a1

          To create a revision using Git source and a metadata file from a local path, run:

            $ {command} my-revision --project=my-project --location=us-central1 \\
            --space=my-space \\
            --catalog=my-catalog \\
            --template=my-template \\
            --git-source-repo=GoogleCloudPlatform/terraform-google-cloud-run \\
            --git-source-ref-tag=v1.0.0 \\
            --git-source-dir=modules/my-product \\
            --metadata=/path/to/metadata.yaml
          """,
    'API REFERENCE': """ \
        This command uses the designcenter/v1alpha API. The full documentation for
        this API can be found at:
        http://cloud.google.com/application-design-center/docs
        """,
}


def _set_git_reference(git_reference_message, ref_string):
  """Populates the correct oneof field in a GitReference message.

  This function parses the reference string to determine if it is a tag,
  branch, or commit. It supports explicit formats (e.g., 'refs/tags/v1.0.0' or
  'refs/heads/modules/my-product' or
  'refs/commits/269b518b99d06b31ff938a2d182e75f5e41941c7').

  Args:
    git_reference_message: The GitReference message to populate.
    ref_string: The user-provided reference string.
  """
  if not ref_string:
    return

  # Check for explicit Git reference formats.
  if ref_string.startswith('refs/tags/'):
    git_reference_message.refTag = ref_string[len('refs/tags/') :]
    return
  if ref_string.startswith('refs/heads/'):
    git_reference_message.branch = ref_string[len('refs/heads/') :]
    return
  if ref_string.startswith('refs/commits/'):
    git_reference_message.commitSha = ref_string[len('refs/commits/') :]
    return

  # If no format matches, raise an error.
  raise exceptions.InvalidArgumentException(
      '--developer-connect-repo-ref',
      'Git reference "{}" does not match the required format: '
      '"refs/tags/<tag>", "refs/heads/<branch>", or "refs/commits/<sha>".'
      .format(ref_string),
  )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.UniverseCompatible
class Create(base.CreateCommand):
  """Create a new catalog template revision."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddCreateCatalogTemplateRevisionFlags(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command."""

    client = utils.GetClientInstance(self.ReleaseTrack())
    messages = utils.GetMessagesModule(self.ReleaseTrack())
    catalog_template_revision = messages.CatalogTemplateRevision()

    revision_ref = args.CONCEPTS.revision.Parse()
    parent = revision_ref.Parent().RelativeName()
    revision_id = revision_ref.Name()

    if (
        args.IsSpecified('git_source_repo')
        or args.IsSpecified('git_source_ref_tag')
        or args.IsSpecified('git_source_dir')
    ):
      catalog_template_revision.gitSource = messages.GitSource(
          repo=args.git_source_repo,
          refTag=args.git_source_ref_tag,
          dir=args.git_source_dir,
      )
    elif (
        args.IsSpecified('developer_connect_repo')
        or args.IsSpecified('developer_connect_repo_ref')
        or args.IsSpecified('developer_connect_repo_dir')
    ):
      git_reference = messages.GitReference()
      # Determine which 'oneof' field to set based on the reference string.
      _set_git_reference(git_reference, args.developer_connect_repo_ref)

      catalog_template_revision.developerConnectSourceConfig = (
          messages.DeveloperConnectSourceConfig(
              developerConnectRepoUri=args.developer_connect_repo,
              reference=git_reference,
              dir=args.developer_connect_repo_dir,
          )
      )
    elif args.application_template_revision_source:
      catalog_template_revision.applicationTemplateRevisionSource = (
          args.application_template_revision_source
      )
    elif args.gcs_source_uri:
      catalog_template_revision.gcsSourceUri = args.gcs_source_uri
    elif args.IsSpecified('oci_repo_uri') or args.IsSpecified(
        'oci_repo_version'
    ):
      catalog_template_revision.ociRepo = messages.OciRepo(
          uri=args.oci_repo_uri,
          version=args.oci_repo_version,
      )

    if args.description:
      catalog_template_revision.description = args.description

    if args.metadata:
      try:
        # The arg type YAMLFileContents() already loads the file.
        metadata_dict = args.metadata
        if 'spec' in metadata_dict:
          catalog_template_revision.metadataInput = messages.MetadataInput(
              spec=messages_util.DictToMessageWithErrorCheck(
                  metadata_dict['spec'], messages.MetadataInputSpec
              )
          )
        else:
          raise exceptions.InvalidArgumentException(
              '--metadata',
              'The metadata file must contain a top-level "spec" key.',
          )
      except yaml.YAMLParseError as e:
        raise exceptions.InvalidArgumentException(
            '--metadata', f'Error parsing YAML file: {e}'
        )

    request = messages.DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsCreateRequest(
        parent=parent,
        catalogTemplateRevisionId=revision_id,
        catalogTemplateRevision=catalog_template_revision,
    )

    operation = (
        client.projects_locations_spaces_catalogs_templates_revisions.Create(
            request
        )
    )
    log.status.Print(
        'Create request issued for: [{0}]'.format(revision_ref.Name())
    )
    if args.async_:
      return operation

    response = utils.WaitForOperationWithEmbeddedResult(
        client=client,
        operation=operation,
        message='Waiting for operation [{0}] to complete'.format(
            operation.name
        ),
        max_wait_sec=7200,
    )
    log.status.Print('Created revision [{0}].'.format(revision_ref.Name()))
    return response


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.UniverseCompatible
class CreateGa(base.CreateCommand):
  """Create a new catalog template revision."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddCreateCatalogTemplateRevisionFlags(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command."""

    client = utils.GetClientInstance(self.ReleaseTrack())
    messages = utils.GetMessagesModule(self.ReleaseTrack())
    catalog_template_revision = messages.CatalogTemplateRevision()

    revision_ref = args.CONCEPTS.revision.Parse()
    parent = revision_ref.Parent().RelativeName()
    revision_id = revision_ref.Name()

    if (
        args.IsSpecified('git_source_repo')
        or args.IsSpecified('git_source_ref_tag')
        or args.IsSpecified('git_source_dir')
    ):
      catalog_template_revision.gitSource = messages.GitSource(
          repo=args.git_source_repo,
          refTag=args.git_source_ref_tag,
          dir=args.git_source_dir,
      )
    elif (
        args.IsSpecified('developer_connect_repo')
        or args.IsSpecified('developer_connect_repo_ref')
        or args.IsSpecified('developer_connect_repo_dir')
    ):
      git_reference = messages.GitReference()
      # Determine which 'oneof' field to set based on the reference string.
      _set_git_reference(git_reference, args.developer_connect_repo_ref)

      catalog_template_revision.developerConnectSourceConfig = (
          messages.DeveloperConnectSourceConfig(
              developerConnectRepoUri=args.developer_connect_repo,
              reference=git_reference,
              dir=args.developer_connect_repo_dir,
          )
      )
    elif args.application_template_revision_source:
      catalog_template_revision.applicationTemplateRevisionSource = (
          args.application_template_revision_source
      )
    elif args.gcs_source_uri:
      catalog_template_revision.gcsSourceUri = args.gcs_source_uri
    elif args.IsSpecified('oci_repo_uri') or args.IsSpecified(
        'oci_repo_version'
    ):
      catalog_template_revision.ociRepo = messages.OciRepo(
          uri=args.oci_repo_uri,
          version=args.oci_repo_version,
      )

    if args.description:
      catalog_template_revision.description = args.description

    if args.metadata:
      try:
        # The arg type YAMLFileContents() already loads the file.
        metadata_dict = args.metadata
        if 'spec' in metadata_dict:
          catalog_template_revision.metadataInput = messages.MetadataInput(
              spec=messages_util.DictToMessageWithErrorCheck(
                  metadata_dict['spec'], messages.MetadataInputSpec
              )
          )
        else:
          raise exceptions.InvalidArgumentException(
              '--metadata',
              'The metadata file must contain a top-level "spec" key.',
          )
      except yaml.YAMLParseError as e:
        raise exceptions.InvalidArgumentException(
            '--metadata', f'Error parsing YAML file: {e}'
        )

    request = messages.DesigncenterProjectsLocationsSpacesCatalogsTemplatesRevisionsCreateRequest(
        parent=parent,
        catalogTemplateRevisionId=revision_id,
        catalogTemplateRevision=catalog_template_revision,
    )

    operation = (
        client.projects_locations_spaces_catalogs_templates_revisions.Create(
            request
        )
    )
    log.status.Print(
        'Create request issued for: [{0}]'.format(revision_ref.Name())
    )
    if args.async_:
      return operation

    response = utils.WaitForOperationWithEmbeddedResult(
        client=client,
        operation=operation,
        message='Waiting for operation [{0}] to complete'.format(
            operation.name
        ),
        max_wait_sec=7200,
    )
    log.status.Print('Created revision [{0}].'.format(revision_ref.Name()))
    return response




# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Set IAP settings."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iap import util as iap_util

EXAMPLES = """\
          To set the IAP setting for the resources within an organization, run:

            $ {command} iap_settings.yaml --organization=ORGANIZATION_ID

          To set the IAP setting for the resources within a folder, run:

            $ {command} iap_settings.yaml --folder=FOLDER_ID

          To set the IAP setting for the resources within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID

          To set the IAP setting for web type resources within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=iap_web

          To set the IAP setting for all app engine services within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=app-engine

          To set the IAP setting for an app engine service within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=app-engine --service=SERVICE_ID

          To set the IAP setting for an app engine service version within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=app-engine --service=SERVICE_ID
                --version=VERSION_ID

          To set the IAP setting for all backend services within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=backend-services

          To set the IAP setting for a backend service within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=backend-services --service=SERVICE_ID

          To set the IAP setting for a region backend service within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=backend-services --service=SERVICE_ID
                --region=REGION_ID

          To set the IAP setting for all forwarding rule within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=forwarding-rule

          To set the IAP setting for a forwarding rule within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=forwarding-rule --service=SERVICE_ID

          To set the IAP setting for a region forwarding rule within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=forwarding-rule --service=SERVICE_ID
              --region=REGION_ID

          To set the IAP setting for the all cloud run services within a region of a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=cloud-run --region=REGION_ID

          To set the IAP setting for a cloud run service within a project, run:

            $ {command} iap_settings.yaml --project=PROJECT_ID --resource-type=cloud-run --region=REGION_ID --service=SERVICE_ID
          """


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.DefaultUniverseOnly
class Set(base.Command):
  """Set the setting for an IAP resource."""

  detailed_help = {
      'EXAMPLES': EXAMPLES,
  }

  @classmethod
  def Args(cls, parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    iap_util.AddIapSettingArg(parser)
    iap_util.AddIapSettingFileArg(parser)
    base.URI_FLAG.RemoveFromParser(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The specified function with its description and configured filter
    """
    iap_setting_ref = iap_util.ParseIapSettingsResource(
        self.ReleaseTrack(),
        args,
    )
    return iap_setting_ref.SetIapSetting(args.setting_file)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Deployment Manager resources sub-group."""


from googlecloudsdk.api_lib.deployment_manager import exceptions
from googlecloudsdk.calliope import base


class Resources(base.Group):
  """Commands for Deployment Manager resources.

  Commands to list and examine resources within a deployment.
  """

  detailed_help = {
      'EXAMPLES': """\
          To view all details about a resource, run:

            $ {command} describe my-resource --deployment my-deployment

          To see the list of all resources in a deployment, run:

            $ {command} list --deployment my-deployment
          """,
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    parser.add_argument('--deployment', help='Deployment name')

  def Filter(self, unused_tool_context, args):
    if not args.deployment:
      raise exceptions.ArgumentError('argument --deployment is required')



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A command that validates YAML data against a JSON Schema."""


from googlecloudsdk.calliope import base
from googlecloudsdk.core import yaml
from googlecloudsdk.core import yaml_validator
from googlecloudsdk.core.console import console_io


class ValidateYAML(base.Command):
  """Validate a YAML file against a JSON Schema.

  {command} validates YAML / JSON files against
  [JSON Schemas](https://json-schema.org/).
  """

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'schema_file',
        help='The path to a file containing the JSON Schema.')
    parser.add_argument(
        'yaml_file',
        help=('The path to a file containing YAML / JSON data. Use `-` for '
              'the standard input.'))

  def Run(self, args):
    contents = console_io.ReadFromFileOrStdin(args.yaml_file, binary=False)
    parsed_yaml = yaml.load(contents)
    yaml_validator.Validator(args.schema_file).Validate(parsed_yaml)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud dataproc-gdc instances create` command."""


import argparse

from apitools.base.py import encoding
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataproc_gdc.spark_applications import basecreate as baseSparkApplication
from googlecloudsdk.command_lib.util.args import labels_util

DATAPROCGDC_API_NAME = 'dataprocgdc'
VERSION_MAP = {
    base.ReleaseTrack.ALPHA: 'v1alpha1',
    base.ReleaseTrack.GA: 'v1',
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Spark(baseSparkApplication.BaseGDCSparkApplicationCommand):
  """Create a Dataproc GDC spark application.

  A spark application that run locally on the Dataproc
  GDC cluster.
  """

  detailed_help = {'EXAMPLES': """\
          To create a Dataproc GDC spark application  with name
          `my-application` in location `us-central1` running on a service
          instance `my-instance`, run:

          $ {command} my-application --service-instance=my-instance
          --location=us-central1 --project=test-project

          """}

  @staticmethod
  def Args(parser):
    baseSparkApplication.BaseGDCSparkApplicationCommand.Args(parser)
    parser.add_argument(
        'job_args',
        nargs=argparse.REMAINDER,
        help='Arguments to pass to the driver.')
    parser.add_argument(
        '--jar',
        help='The HCFS URI of the jar file that contains the main class.',
    )
    parser.add_argument(
        '--main-class',
        help=(
            'The name of the driver main class. The jar file that contains'
            'the class must be in the classpath or specified in jar_file_uris'
        ),
    )
    parser.add_argument(
        '--jars',
        type=arg_parsers.ArgList(),
        metavar='JAR',
        default=[],
        help=('Comma separated list of jar files to be provided to the '
              'executor and driver classpaths.'))
    parser.add_argument(
        '--files',
        type=arg_parsers.ArgList(),
        metavar='FILE',
        default=[],
        help=('Comma separated list of files to be placed in the working '
              'directory of both the app driver and executors.'))
    parser.add_argument(
        '--archives',
        type=arg_parsers.ArgList(),
        metavar='ARCHIVE',
        default=[],
        help=(
            'Comma separated list of archives to be extracted into the working '
            'directory of each executor. '
            'Must be one of the following file formats: .zip, .tar, .tar.gz, '
            'or .tgz.'))

  def Run(self, args):
    api_version = VERSION_MAP.get(self.ReleaseTrack())
    messages = apis.GetMessagesModule(DATAPROCGDC_API_NAME, api_version)
    application_ref = args.CONCEPTS.application.Parse()
    application_environment_ref = args.CONCEPTS.application_environment.Parse()
    service_instance_ref = args.CONCEPTS.service_instance.Parse()

    if args.annotations:
      annotations = encoding.DictToAdditionalPropertyMessage(
          args.annotations,
          messages.SparkApplication.AnnotationsValue,
          sort_items=True,
      )
    else:
      annotations = None

    application_environment = None
    if application_environment_ref:
      application_environment = application_environment_ref.Name()

    spark_app_properties = None

    if args.properties:
      spark_app_properties = encoding.DictToAdditionalPropertyMessage(
          args.properties,
          messages.SparkApplication.PropertiesValue,
      )

    create_req = messages.DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsCreateRequest(
        parent=service_instance_ref.RelativeName(),
        sparkApplication=messages.SparkApplication(
            applicationEnvironment=application_environment,
            displayName=args.display_name,
            labels=labels_util.ParseCreateArgs(
                args, messages.SparkApplication.LabelsValue
            ),
            annotations=annotations,
            namespace=args.namespace,
            properties=spark_app_properties,
            version=args.version,
            dependencyImages=args.dependency_images or [],
            sparkApplicationConfig=messages.SparkApplicationConfig(
                args=args.job_args or [],
                mainJarFileUri=args.jar,
                fileUris=args.files or [],
                jarFileUris=args.jars or [],
                mainClass=args.main_class,
                archiveUris=args.archives or [],
            ),
        ),
    )
    super().Submit(args, application_ref, create_req)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command used for updating resource maps."""


from googlecloudsdk.calliope import base

from googlecloudsdk.command_lib.util.resource_map import resource_map_update_util
from googlecloudsdk.command_lib.util.resource_map.declarative import declarative_map_update_util


class UpdateResourceMap(base.Command):
  """Command used for updating resource maps."""

  @staticmethod
  def Args(parser):
    group = parser.add_group(mutex=True)
    group.add_argument(
        '--all', action='store_true', help='Update all resource maps.')
    individual_maps_group = group.add_group()
    individual_maps_group.add_argument(
        '--declarative',
        action='store_true',
        help='Update the declarative resource map.')
    individual_maps_group.add_argument(
        '--base', action='store_true', help='Update the base resource map.')

  def Run(self, args):
    if args.base or args.all:
      resource_map_update_util.update()
    if args.declarative or args.all:
      declarative_map_update_util.update()



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list long-running operations."""


from typing import Generator

from googlecloudsdk.api_lib.container.fleet import client
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_arguments
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.command_lib.container.fleet import flags as fleet_flags
from googlecloudsdk.command_lib.container.fleet import util as fleet_util
from googlecloudsdk.generated_clients.apis.gkehub.v1alpha import gkehub_v1alpha_messages as messages


_EXAMPLES = """
To list all operations in location ``us-west1'', run:

$ {command} --location=us-west1
"""


class List(base.ListCommand):
  """List long-running operations."""

  detailed_help = {'EXAMPLES': _EXAMPLES}

  @staticmethod
  def Args(parser: parser_arguments.ArgumentInterceptor) -> None:
    """Registers flags for this command.

    Args:
      parser: Top level argument group to add new arguments.
    """
    flags = fleet_flags.FleetFlags(parser)
    flags.AddLocation()

  def Run(
      self, args: parser_extensions.Namespace
  ) -> Generator[messages.Operation, None, None]:
    """Runs the operations list command.

    Long-running operations are retained no more than a week in the server.

    Args:
      args: Flag arguments received from command line.

    Returns:
      A list of long-running operations under the project.
    """
    if '--format' not in args.GetSpecifiedArgNames():
      args.format = fleet_util.OPERATION_FORMAT

    flag_parser = fleet_flags.FleetFlagParser(
        args, release_track=self.ReleaseTrack()
    )
    operation_client = client.OperationClient(self.ReleaseTrack())

    req = flag_parser.messages.GkehubProjectsLocationsOperationsListRequest(
        name=util.LocationResourceName(
            flag_parser.Project(), location=flag_parser.Location()
        )
    )
    return operation_client.List(
        req, page_size=flag_parser.PageSize(), limit=flag_parser.Limit()
    )
